/*
 * xsky - a computerized sky atlas
 *
 * Copyright 1992-3, Terry R. Friedrichsen
 *
 * This program may be copied and redistributed, in whole or in part,
 * as long as you don't try to make any money from the sale or redis-
 * tribution of the program or any part of the program, or pretend
 * that you wrote the program or any of its parts unless specifically
 * credited by the original author.
 *
 * You are free to make use of this software in your own programs, as
 * long as you credit the original author where it is due.
 */

/*
 * WARRANTY:
 * xsky was written as a learning project and as a demonstration of
 * X Window System programming.  xsky doesn't do anything; it is not
 * merchantable, and it is not fit for any purpose whatsoever.  In
 * fact, don't use xsky at all; it's free, and you're getting what
 * you paid for.
 */

#include <stdio.h>

#include <X11/Intrinsic.h>

#include "skydefs.h"
#include "catalog.h"
#include "draw.h"

#ifdef __STDC__
/* global function prototypes */
void draw_id_text(Widget,char *,IDfont,int,int);
boolean ps_put(char *);
#else   /* not __STDC__ */
/* global function prototypes */
void draw_id_text();
boolean ps_put();
#endif   /* __STDC__ */

extern int blackpixelval;
extern int whitepixelval;

extern GC star_gcs[];
extern GC mono_star_gc;
extern GC obj_gc;
extern GC text_gc;
extern GC grid_gc;

extern XFontStruct *latinidfontstr;
extern XFontStruct *greekidfontstr;

extern boolean mono_flag;

extern int x_height;
extern int x_width;

/* PostScript output file */
FILE *psfile;

/* flag for new path for grid line */
static boolean ps_new_path;

/* buffer for generated PostScript lines */
static char ps_buffer[200];

/* count of the number of header lines generated */
static int headerlines;



/* draw a single star on the sky */

void draw_star(sky_widget,star_x,star_y,star_diam,color)

Widget sky_widget;
Position star_x, star_y;
int star_diam;
StarColor color;

{
  Display *sky_display;
  Window sky_window;
  Position ul_x, ul_y;

#ifdef DEBUG
  printf("draw_star called to draw size %d dot at %d,%d\n",
	                                             star_diam,star_x,star_y);
#endif

/* get the display for the widget */
  sky_display = XtDisplay(sky_widget);

/* get the window for the widget */
  sky_window = XtWindow(sky_widget);

/* calculate the upper-left coordinates of the enclosing box */
  ul_x = star_x - star_diam / 2;
  ul_y = star_y - star_diam / 2;

/* draw a star */
  if (mono_flag)
    /* monochrome display */
    XFillArc(sky_display,sky_window,mono_star_gc,
	                                     ul_x,ul_y,
		                             star_diam,star_diam,
	                                     0,360 * 64);
  else
    /* color display */
    XFillArc(sky_display,sky_window,star_gcs[color],
	                                     ul_x,ul_y,
		                             star_diam,star_diam,
	                                     0,360 * 64);

  return;
}



/* draw a single object on the sky */

void draw_object(sky_widget,obj_x,obj_y,obj_size,shape)

Widget sky_widget;
Position obj_x, obj_y;
int obj_size;
ObjShape shape;

{
  Display *sky_display;
  Window sky_window;
  Position ul_x, ul_y;

/* get the display for the widget */
  sky_display = XtDisplay(sky_widget);

/* get the window for the widget */
  sky_window = XtWindow(sky_widget);

/* calculate the upper-left coordinates of the enclosing box */
  ul_x = obj_x - obj_size / 2;
  ul_y = obj_y - obj_size / 2;

/* draw an object figure depending on the given shape */
  switch (shape) {
  case ELLIPSE:
    XDrawArc(sky_display,sky_window,obj_gc,ul_x,ul_y,
	                                    obj_size + 2,obj_size,0,360 * 64);
    break;
  case CIRCLE:
    XDrawArc(sky_display,sky_window,obj_gc,ul_x,ul_y,
	                                obj_size + 1,obj_size + 1,0,360 * 64);
    break;
  case SQUARE:
    XDrawRectangle(sky_display,sky_window,obj_gc,ul_x,ul_y,obj_size,obj_size);
    break;
  case XMARK:
    draw_id_text(sky_widget,"x",LATIN,(Position)(obj_x - x_width / 2),
		                            (Position)(obj_y + x_height / 2));
    break;
  default:
    break;
  }

  return;
}



/* display the ID text for the object */


void draw_id_text(sky_widget,buff,font,x,y)

Widget sky_widget;
char *buff;
IDfont font;
Position x, y;

{
  Display *sky_display;
  Window sky_window;

/* get the display for the widget */
  sky_display = XtDisplay(sky_widget);

/* get the window for the widget */
  sky_window = XtWindow(sky_widget);

/* set the correct font */
  if (font == LATIN)
    XSetFont(XtDisplay(sky_widget),text_gc,latinidfontstr->fid);
  else if (font == GREEK)
    XSetFont(XtDisplay(sky_widget),text_gc,greekidfontstr->fid);
  else
    /* default to Latin */
    XSetFont(XtDisplay(sky_widget),text_gc,latinidfontstr->fid);

/* set the foreground to white to draw the text characters */
  XSetForeground(sky_display,text_gc,whitepixelval);

/* use the copy function to scribble over any overlaid objects */
  XSetFunction(sky_display,text_gc,GXcopy);

/* and draw the ID string */
  XDrawImageString(sky_display,sky_window,text_gc,x,y,buff,strlen(buff));

  return;
}



/* draw a dot on the X display */

void draw_point(sky_widget,x,y)

Widget sky_widget;
Position x, y;

{
  XDrawPoint(XtDisplay(sky_widget),XtWindow(sky_widget),grid_gc,x,y);

  return;
}



/* output the PostScript function definitions */

boolean ps_setup()

{
/* start out the PostScript file in a standard way */
  if (! ps_put("%!PS-Adobe-2.0\n"))
    return(FALSE);

/* define a function to draw an ellipse */
  if (! ps_put("/ellipse {\n"))
    return(FALSE);
  if (! ps_put("    factor div 2 div\n"))
    return(FALSE);
  if (! ps_put("    gsave\n"))
    return(FALSE);
  if (! ps_put("    3 1 roll 2 mul 3 -1 roll 1 .5 scale\n"))
    return(FALSE);
  if (! ps_put("    newpath 0 360 arc closepath\n"))
    return(FALSE);
  if (! ps_put("    .3 setlinewidth 0 setgray stroke\n"))
    return(FALSE);
  if (! ps_put("    grestore\n"))
    return(FALSE);
  if (! ps_put("} bind def\n"))
    return(FALSE);

/* define a function to draw a circle */
  if (! ps_put("/circle {\n"))
    return(FALSE);
  if (! ps_put("    factor div 2 div\n"))
    return(FALSE);
#if 0
  if (! ps_put("    dup 4 1 roll sub exch 2 index add exch 3 -1 roll\n"))
    return(FALSE);
#endif
  if (! ps_put("    newpath 0 360 arc closepath\n"))
    return(FALSE);
  if (! ps_put("    gsave .3 setlinewidth 0 setgray stroke grestore\n"))
    return(FALSE);
  if (! ps_put("} bind def\n"))
    return(FALSE);

/* define a function to draw a box for object location */
  if (! ps_put("/box {\n"))
    return(FALSE);
  if (! ps_put("    factor div\n"))
    return(FALSE);
  if (! ps_put("    3 1 roll\n"))
    return(FALSE);
  if (! ps_put("    newpath moveto\n"))
    return(FALSE);
  if (! ps_put("    dup neg 0 exch rlineto\n"))
    return(FALSE);
  if (! ps_put("    dup 0 rlineto\n"))
    return(FALSE);
  if (! ps_put("    dup 0 exch rlineto\n"))
    return(FALSE);
  if (! ps_put("    dup neg 0 rlineto\n"))
    return(FALSE);
  if (! ps_put("    closepath\n"))
    return(FALSE);
  if (! ps_put("    gsave .3 setlinewidth 0 setgray stroke grestore\n"))
    return(FALSE);
  if (! ps_put("} bind def\n"))
    return(FALSE);

/* define a function to draw a filled-in circle for a star */
  if (! ps_put("/star {\n"))
    return(FALSE);
  if (! ps_put("    factor div 2 div\n"))
    return(FALSE);
  if (! ps_put("    newpath 0 360 arc closepath\n"))
    return(FALSE);
  if (! ps_put("    gsave .3 setlinewidth 1 setgray stroke grestore\n"))
    return(FALSE);
  if (! ps_put("    0 setgray fill\n"))
    return(FALSE);
  if (! ps_put("} bind def\n"))
    return(FALSE);

/* define a function to emit a character at an object position */
  if (! ps_put("/put_obj_char {\n"))
    return(FALSE);
  if (! ps_put("    gsave\n"))
    return(FALSE);
  if (! ps_put("    Latin setfont\n"))
    return(FALSE);
  if (! ps_put("    dup stringwidth pop 2 div\n"))
    return(FALSE);
  if (! ps_put("    4 -1 roll exch sub\n"))
    return(FALSE);
  /* adjusting the y value by 1/6 the font scale is purely empirical */
  if (! ps_put("    3 -1 roll Latinfontscale 6 div sub moveto\n"))
    return(FALSE);
  if (! ps_put("    1 factor div dup scale\n"))
    return(FALSE);
  if (! ps_put("    show\n"))
    return(FALSE);
  if (! ps_put("    grestore\n"))
    return(FALSE);
  if (! ps_put("} bind def\n"))
    return(FALSE);

#if 0
/* define a function to emit object ID text */
  if (! ps_put("/put_id {\n"))
    return(FALSE);
  if (! ps_put("    gsave\n"))
    return(FALSE);
  if (! ps_put("    dup stringwidth pop 3 -1 roll exch div\n"))
    return(FALSE);
  if (! ps_put("    4 -2 roll moveto\n"))
    return(FALSE);
  if (! ps_put("    1 factor div scale\n"))
    return(FALSE);
  if (! ps_put("    show\n"))
    return(FALSE);
  if (! ps_put("    grestore\n"))
    return(FALSE);
  if (! ps_put("} bind def\n"))
    return(FALSE);
#else
/* define a function to emit object ID text */
  if (! ps_put("/put_corr_id {\n"))
    return(FALSE);
  if (! ps_put("    6 1 roll\n"))
    return(FALSE);
  if (! ps_put("    setfont\n"))
    return(FALSE);
  if (! ps_put("    dup factor .8 mul div sub sub\n"))
    return(FALSE);
  if (! ps_put("    4 1 roll dup factor div sub add\n"))
    return(FALSE);
  if (! ps_put("    3 -1 roll moveto\n"))
    return(FALSE);
  if (! ps_put("    show\n"))
    return(FALSE);
  if (! ps_put("} bind def\n"))
    return(FALSE);
#endif

  return(TRUE);
}



/* set up to begin chart header */

boolean ps_begin_header()

{
/* capture the boundaries of the printable area */
  if (! ps_put("gsave initclip clippath pathbbox grestore\n"))
    return(FALSE);
  if (! ps_put("/ur_y exch def /ur_x exch def\n"))
    return(FALSE);
  if (! ps_put("/ll_y exch def /ll_x exch def\n"))
    return(FALSE);

/* set up the Symbol font for Greek letters */
  if (! ps_put("/Times-Roman findfont 18 scalefont setfont\n"))
    return(FALSE);

/* draw the top line of the header box */
  if (! ps_put("newpath ll_x 1 add ur_y 2 sub moveto\n"))
    return(FALSE);
  if (! ps_put("ur_x ll_x sub 2 sub 0 rlineto\n"))
    return(FALSE);
  if (! ps_put("3 setlinewidth stroke\n"))
    return(FALSE);

/* set up the y position for the first header line */
  if (! ps_put("/curr_y ur_y 22 sub def\n"))
    return(FALSE);

/* initialize the header line count */
  headerlines = 0;

  return(TRUE);
}



/* put out lines of header information */

boolean ps_put_header(line)

char *line;

{
  char *ptr1, *ptr2;

/* draw the left-hand box segment */
  if (! ps_put("newpath ll_x 2 add curr_y 8 sub moveto\n"))
    return(FALSE);
  if (! ps_put("0 28 rlineto 3 setlinewidth stroke\n"))
    return(FALSE);

/* draw the right-hand box segment */
  if (! ps_put("newpath ur_x 2 sub curr_y 8 sub moveto\n"))
    return(FALSE);
  if (! ps_put("0 28 rlineto 3 setlinewidth stroke\n"))
    return(FALSE);

/* move to the text position */
  if (! ps_put("ll_x 10 add curr_y moveto\n"))
    return(FALSE);

/* set up source and destination pointers */
  ptr1 = line;
  ptr2 = ps_buffer;

/* put the user line in the buffer, escaping parens and backslashes */
  *ptr2++ = '(';
  do {
    if ((*ptr1 == '(') || (*ptr1 == ')') || (*ptr1 == '\\'))
      *ptr2++ = '\\';
    /* replace twiddles with a degree symbol */
    if (*ptr1 == '~') {
      /* end the current line here */
      strcpy(ptr2,") show\n");
      if (! ps_put(ps_buffer))
	return(FALSE);
      /* put out a degree symbol */
      if (! ps_put("0 7 rmoveto (o) show 0 -7 rmoveto\n"))
	return(FALSE);
      /* restart the line */
      ptr2 = ps_buffer;
      *ptr2++ = '(';
      /* step past the twiddle */
      ptr1++;
      /* check for escapeable characters following the twiddle */
      if ((*ptr1 == '(') || (*ptr1 == ')') || (*ptr1 == '\\'))
	*ptr2++ = '\\';
    }
  } while ((*ptr2++ = *ptr1++) != '\0');
  
/* end the line properly */
  strcat(ps_buffer,") show\n");

/* and put it out */
  if (! ps_put(ps_buffer))
    return(FALSE);

/* move to the next line position */
  if (! ps_put("/curr_y curr_y 20 sub def\n"))
    return(FALSE);

/* count another line of header information */
  headerlines++;

  return(TRUE);
}



/* end the header information */

boolean ps_end_header()

{
/* draw the bottom line of the header box */
  if (! ps_put("newpath ll_x 1 add curr_y 12 add moveto\n"))
    return(FALSE);
  if (! ps_put("ur_x ll_x sub 2 sub 0 rlineto\n"))
    return(FALSE);
  if (! ps_put("3 setlinewidth stroke\n"))
    return(FALSE);

  return(TRUE);
}



/* set up to begin drawing commands */

boolean ps_begin_drawing(width,height)

Dimension width, height;

{
/* set up the Symbol font for Greek letters */
  if (! ps_put("/Greekfontscale 12 def\n"))
    return(FALSE);
  if (! ps_put("/Greek /Symbol findfont Greekfontscale scalefont def\n"))
    return(FALSE);

/* set up the Times-Roman font for Latin designations */
  if (! ps_put("/Latinfontscale 12 def\n"))
    return(FALSE);
  if (! ps_put("/Latin /Times-Roman findfont Latinfontscale scalefont def\n"))
    return(FALSE);

/* set a narrow line width and rounded line join style for grid lines */
  if (! ps_put("0 setlinewidth 1 setlinejoin\n"))
    return(FALSE);

/* compute the correct scale factor */
  sprintf(ps_buffer,"/factor_x ur_x ll_x sub %d div def\n",width);
  if (! ps_put(ps_buffer))
    return(FALSE);
  sprintf(ps_buffer,"/factor_y ur_y ll_y sub %d sub %d div def\n",
	                                        headerlines * 20 + 15,height);
  if (! ps_put(ps_buffer))
    return(FALSE);
  if (! ps_put("/factor factor_x factor_y lt\n"))
    return(FALSE);
  if (! ps_put("    { factor_x } { factor_y } ifelse def\n"))
    return(FALSE);

/* scale the drawing to the correct size */
  if (! ps_put("factor factor scale\n"))
    return(FALSE);

/* translate coordinates to the imageable lower left */
  if (! ps_put("ll_x ll_y translate\n"))
    return(FALSE);

  return(TRUE);
}



/* end a PostScript drawing */

boolean ps_end_drawing()

{
  return(ps_put("showpage\n\004\n"));
}



/* draw an object ID for PostScript */

boolean ps_draw_id(id,height)

struct id_node *id;
Dimension height;

{
#if 0
  float x, y;
#endif
  float x_corr, y_corr;
  char *font;

#ifdef DEBUG
  printf("object is %s at %d,%d\n",id->id_buffer,id->obj_x,id->obj_y);
  printf("ID text is at %d,%d\n",id->x,id->y);
  printf("ID upper left is at %d,%d\n",id->ul_x,id->ul_y);
  printf("ID lower right is at %d,%d\n",id->lr_x,id->lr_y);
  printf("therefore text length = %d\n",id->lr_x - id->ul_x);
#endif

#if 0
/* calculate where to place the text */
  if (id->obj_x < id->x)
    /* apply this correction if the ID is to the right of the object */
    x = id->x - ((id->ul_x - id->obj_x) - (id->ul_x - id->obj_x) / factor);
  else if (id->obj_x > id->x)
    /* apply this correction if the ID is to the left of the object */
    x = id->x + ((id->obj_x - id->lr_x) - (id->obj_x - id->lr_x) / factor);
  else
    /* ID coincides with object horizontally; don't correct position */
    x = id->x;
#endif

/* calculate correction to text location */
  if (id->obj_x < id->x)
    /* apply this correction if the ID is to the right of the object */
    x_corr = -(id->ul_x - id->obj_x);
/*    x_corr = -((id->ul_x - id->obj_x) - (id->ul_x - id->obj_x));*/
  else if (id->obj_x > id->x)
    /* apply this correction if the ID is to the left of the object */
    x_corr = id->obj_x - id->lr_x;
/*    x_corr = ((id->obj_x - id->lr_x) - (id->obj_x - id->lr_x));*/
  else
    /* ID coincides with object horizontally; don't correct position */
    x_corr = 0;

#if 0
/* correct the y coordinate in the same manner */
  if (id->obj_y < id->y)
    /* apply this correction if the ID is below the object */
    y = id->y - ((id->y - id->obj_y) - (id->y - id->obj_y) / factor);
  else if (id->obj_y > id->y)
    /* apply this correction if the ID is above the object */
    y = id->y + ((id->obj_y - id->y) - (id->obj_y - id->y) / factor);
  else
    /* ID coincides with object vertically; don't correct position */
    y = id->y;
#endif

/* calculate the y coordinate correction in the same manner */
  if (id->obj_y < id->y)
    /* apply this correction if the ID is below the object */
    y_corr = -(id->y - id->obj_y);
/*    y_corr = -((id->y - id->obj_y) - (id->y - id->obj_y));*/
  else if (id->obj_y > id->y)
    /* apply this correction if the ID is above the object */
    y_corr = id->obj_y - id->y;
/*    y_corr = ((id->obj_y - id->y) - (id->obj_y - id->y));*/
  else
    /* ID coincides with object vertically; don't correct position */
    y_corr = 0;

#if 0
#ifdef DEBUG
  printf("corrected x value is %f\n",x);
  printf("corrected y value is %f\n\n",y);
#endif
#else
#ifdef DEBUG
  printf("x correction is %f\n",x_corr);
  printf("y correction is %f\n\n",y_corr);
#endif
#endif

/* set up a recognizeable font string */
  if (id->font == LATIN)
    font = "Latin";
  else if (id->font == GREEK)
    font = "Greek";
  else
    /* default to Latin font */
    font = "Latin";

/* build the call to the ID display routine and write it out */
#if 0
  sprintf(ps_buffer,"%f %f %d (%s) put_id\n",x,height - y,
	                                   id->lr_x - id->ul_x,id->id_buffer);
#else
  sprintf(ps_buffer,"%f %f %f %f %s (%s) put_corr_id\n",(float)id->x,x_corr,
	                                   (float)height - id->y,y_corr,
	                                                  font,id->id_buffer);
#endif
  if (! ps_put(ps_buffer))
    return(FALSE);

  return(TRUE);
}



/* start a new path for a line of right ascension or declination */

void ps_grid_newpath()

{
/* set a flag and emit a newpath command */
  ps_new_path = TRUE;
  (void)ps_put("newpath\n");

  return;
}



/* end the path for a line of right ascension or declination */

void ps_grid_endpath()

{
/* stroke the line just produced */
  (void)ps_put("stroke\n");

  return;
}



/* draw a star for PostScript */

boolean ps_draw_star(x,y,height,size)

double x, y;
Dimension height;
double size;

{
/* generate the line of PostScript to call the filled-circle routine */
  sprintf(ps_buffer,"%.3f %.3f %.3f star\n",x,height - y,size);

/* return the result of outputting the buffer */
  return(ps_put(ps_buffer));
}



/* draw a single object on the sky */

boolean ps_draw_object(x,y,height,size,shape)

double x, y;
Dimension height;
double size;
ObjShape shape;

{
/* draw an object figure depending on the given shape */
  switch (shape) {
  case ELLIPSE:
    sprintf(ps_buffer,"%.3f %.3f %.3f ellipse\n",x,height - y,size + 2);
    break;
  case CIRCLE:
    sprintf(ps_buffer,"%.3f %.3f %.3f circle\n",x,height - y,size + 1);
    break;
  case SQUARE:
    sprintf(ps_buffer,"%.3f %.3f %.3f box\n",x,height - y,size);
    break;
  case XMARK:
    sprintf(ps_buffer,"%.3f %.3f (%s) put_obj_char\n",x,height - y,"x");
    break;
  default:
    break;
  }

/* return the result of outputting the buffer */
  return(ps_put(ps_buffer));
}



/* extend a line to a new point on the PostScript display */

void ps_draw_point(x,y,height)

double x, y;
Dimension height;

{
/* if this is the start of a new path, we need a moveto */
  if (ps_new_path) {
    sprintf(ps_buffer,"%.3f %.3f moveto\n",x,height - y);
    ps_new_path = FALSE;
  }
  else
    /* otherwise, just extend the line to the new point */
    sprintf(ps_buffer,"%.3f %.3f lineto\n",x,height - y);

  (void)ps_put(ps_buffer);

  return;
}



/* output a string to the PostScript file */

boolean ps_put(string)

char *string;

{
  if (fputs(string,psfile) == EOF) {
    fclose(psfile);
    return(FALSE);
  }
  else
    return(TRUE);

/* NOTREACHED */
}
