/*
 * xsky - a computerized sky atlas
 *
 * Copyright 1992-3, Terry R. Friedrichsen
 *
 * This program may be copied and redistributed, in whole or in part,
 * as long as you don't try to make any money from the sale or redis-
 * tribution of the program or any part of the program, or pretend
 * that you wrote the program or any of its parts unless specifically
 * credited by the original author.
 *
 * You are free to make use of this software in your own programs, as
 * long as you credit the original author where it is due.
 */

/*
 * WARRANTY:
 * xsky was written as a learning project and as a demonstration of
 * X Window System programming.  xsky doesn't do anything; it is not
 * merchantable, and it is not fit for any purpose whatsoever.  In
 * fact, don't use xsky at all; it's free, and you're getting what
 * you paid for.
 */

/* this routine prints out English for the new-style RNGC object
 * descriptions.  the algoritm is to grab each piece delimited
 * by commas, then match the longest possible piece with the
 * RNGC description code table.  that piece's translation is
 * then output and the piece is discarded. */

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "rngc.h"

#include "skydefs.h"

#include "rngc_desc.h"

#include "format.h"

#ifdef __STDC__
/* static function prototypes */
static int find_longest_match(char *,int *,struct key_desc *);

/* external function prototypes */
extern char *build_filespec(char *,char *);
#else   /* not __STDC */
/* static function prototypes */
static int find_longest_match();

/* external function prototypes */
extern char *build_filespec();
#endif   /* __STDC__ */

static struct key_desc desc_table[MAXNUMDESCS + 1];

static int max_desc;

#define	MAXBUFF		(MAX_DESC_NAME + MAX_DESC_STRING + 10)



void display_desc(record)

char *record;

{
/* record is a pointer to the start of the RNGC description text */
  int i, j;
  char descbuff[MAXDESCLEN + 1];
  int descidx;
  short nomatch_flag;

/* strip trailing blanks from the record */
  i = strlen(record) - 1;
  while (isspace(record[--i]))
    if (i == 0) {
      i--;
      break;
    }
  record[++i] = '\0';

/* output a header */
  add_string("Revised NGC description:  ");

/* show the actual raw description string */
  add_string(record);

/* position for start of expanded description text */
  blank_line();
  advance_to_tab();

/* assume we will find matches */
  nomatch_flag = FALSE;

/* loop through the description buffer */
  i = 0;
  while (TRUE) {
    /* skip any white space to get to descriptions */
    while (isspace(record[i]))
      i++;

    /* stop if we reach the null */
    if (record[i] == '\0')
      break;

/* copy a description piece until a comma, blank, or null */
    j = 0;
    while ((record[i] != ',') && (record[i] != '\0') && (! isspace(record[i])))
      descbuff[j++] = record[i++];
    /* end the description buffer with a null */
    descbuff[j] = '\0';

/* now loop through the description, piecing out the code words */
    j = 0;
    while (j < strlen(descbuff)) {
/* now match the largest piece possible from the description table */
      descidx = find_longest_match(descbuff,&j,desc_table);

/* advance to the next line, if necessary (leaving room for a delimiter) */
      if (((idx - row_start) + strlen(desc_table[descidx].desc_string) + 1) >
	                                                               max_row)
	next_line();

/* put out the expansion of this piece of the description */
      if ((descidx == max_desc) && nomatch_flag)
	/* no, and neither did the last one; print with no preceding space */
	;
      else if (desc_table[descidx].desc_string[0] == '?')
	/* also, don't space in front of a question mark */
	;
      else
	/* precede the piece of description by a space */
	add_string(" ");
      add_string(desc_table[descidx].desc_string);

/* set nomatch flag according to whether this piece matched or not */
      nomatch_flag = (descidx == max_desc);
    }

/* put out the delimiter unless it is a blank or a null */
    if (record[i] == '\0')
      /* we are done when we see the null */
      break;
    else if (record[i] == ' ')
      if (nomatch_flag) {
	/* put out the delimiter anyway */
	sprintf(&obj_info[idx],"%c",record[i]);
	idx += strlen(&obj_info[idx]);
      }
      else
	;
    else {
      sprintf(&obj_info[idx],"%c",record[i]);
      idx += strlen(&obj_info[idx]);
    }

/* step past the delimiter to the next piece of the description */
    i++;
  }

  return;
}



void read_desc_table()

{
  char *filebuff;
  FILE *desc_fd;
  char inbuf[MAXBUFF + 1];
  int desc;
  int i;

/* build the file path */
  filebuff = build_filespec("RNGC_PATH",RNGC_DESC_FILE);

/* open the descriptor file */
  if ((desc_fd = fopen(filebuff,"r")) == (FILE *)NULL) {
    printf("sky:  error opening RNGC descriptor file %s\n",filebuff);
    perror("sky");
    exit(1);
  }

/* loop through the file, reading in descriptor table entries */
  desc = 0;
  while (fgets(inbuf,MAXBUFF,desc_fd) != NULL) {
    /* remove the newline at the end */
    inbuf[strlen(inbuf) - 1] = '\0';

/* copy the description abbreviation until white space */
    i = 0;
    while (! isspace(inbuf[i])) {
      desc_table[desc].desc_name[i] = inbuf[i];
      i++;
    }

    /* be sure the abbreviation ends with a null */
    desc_table[desc].desc_name[i] = '\0';

/* skip to the description string */
    while (isspace(inbuf[i]))
      i++;

/* copy in the description string */
    strcpy(desc_table[desc].desc_string,&inbuf[i]);

/* step to the next table entry */
    desc++;
  }

/* be neat */
  (void)fclose(desc_fd);

/* free the filename buffer */
  free(filebuff);

/* make the last description be some question marks */
  strcpy(desc_table[desc].desc_name,"\\");
  strcpy(desc_table[desc].desc_string,"???");

/* remember the number of descriptions read */
  max_desc = desc;

  return;
}



static int find_longest_match(buff,ind,table)

char *buff;
int *ind;
struct key_desc *table;

{
  int max_match, max_match_idx;
  char *ptr;
  int desc;
  int i;

/* make the match length impossibly small */
  max_match = -1;
  max_match_idx = -1;

/* begin at the beginning */
  desc = 0;
  while (desc <= max_desc) {

/* point to the start of the description */
    ptr = &buff[*ind];

/* match characters for a while */
    i = strlen(table[desc].desc_name);
    if (strncmp(ptr,table[desc].desc_name,(size_t)i) == EQUAL)
      if (i > max_match) {

/* the new match is bigger; save the length and the index */
	max_match = i;
	max_match_idx = desc;
      }

/* step to the next description */
    if (table[++desc].desc_name[0] == '\\')
      break;
  }

/* if any match was found, update the index by the 1-relative length */
  if (max_match_idx != -1)
    *ind += max_match;
  else {
/* update by one to bypass a bogus piece of description and set up the
 * unrecognized character as the output */
    table[desc].desc_string[0] = buff[(*ind)++];
    table[desc].desc_string[1] = '\0';

/* make the matching index be this one */
    max_match_idx = desc;
  }

/* return the index to the relevant description table entry */
  return(max_match_idx);
}
