/*
 * xsky - a computerized sky atlas
 *
 * Copyright 1992-3, Terry R. Friedrichsen
 *
 * This program may be copied and redistributed, in whole or in part,
 * as long as you don't try to make any money from the sale or redis-
 * tribution of the program or any part of the program, or pretend
 * that you wrote the program or any of its parts unless specifically
 * credited by the original author.
 *
 * You are free to make use of this software in your own programs, as
 * long as you credit the original author where it is due.
 */

/*
 * WARRANTY:
 * xsky was written as a learning project and as a demonstration of
 * X Window System programming.  xsky doesn't do anything; it is not
 * merchantable, and it is not fit for any purpose whatsoever.  In
 * fact, don't use xsky at all; it's free, and you're getting what
 * you paid for.
 */

#include <stdio.h>
#include <ctype.h>

#include <math.h>
#include <string.h>

#include <stdlib.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Shell.h>
#include <X11/Core.h>

#include <X11/cursorfont.h>

#include <X11/Xaw/Command.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Text.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/Scrollbar.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Toggle.h>

#include "skydefs.h"
#include "catalog.h"
#include "pos.h"
#include "draw.h"

#include "sky.h"

#ifdef __STDC__
/* static function prototypes */
static void printusage(char *[]);
static void initialize_display();
static void buildmaindisplay();
static Widget makepanelbutton(int,Widget,Widget,XtCallbackProc,XtPointer,
			                                 Widget *,Dimension *);
static Widget makepaneltoggle(int,Widget,Widget,XtCallbackProc,XtPointer,
			                                 Widget *,Dimension *);
static void makecenterfield(Widget,Widget);
static void calculate_field(struct display *);
static void setup_cursors();
static void setup_graphics();
static Boolean buildinfodisplay();
static Boolean buildcatalogmenu();
static Boolean buildfinddisplay();
static Boolean buildobjdisplay();
static Widget maketextfield(Widget,Widget *,XFontStruct *,Pixel,
			                                    char *,char *,int);
static XtCallbackProc PrepareInfo(Widget,XtPointer,XtPointer);
static XtCallbackProc PrepareFind(Widget,XtPointer,XtPointer);
static Boolean make_big_initial(XtPointer);
static XtCallbackProc handle_grid_button(Widget,XtPointer,XtPointer);
static XtCallbackProc toggle_ids(Widget,XtPointer,XtPointer);
static XtCallbackProc erase_ids(Widget,XtPointer,XtPointer);
static XtCallbackProc popup_cat_menu(Widget,XtPointer,XtPointer);
static XtCallbackProc ps_draw_sky(Widget,XtPointer,XtPointer);
static XtCallbackProc Quit(Widget,XtPointer,XtPointer);
static void set_normal_cursor();
static void set_wait_cursor();
static void set_textbox_cursor();
static void draw_sky(int,int,int,int);
static boolean ps_write_file();
static XtEventHandler cancel_dpyinfo(Widget,XtPointer,XEvent *,Boolean *);
static XtCallbackProc DpyInfoAcked(Widget,XtPointer,XtPointer);
static XtCallbackProc DataAcked(Widget,XtPointer,XtPointer);
static XtEventHandler DataAckEvent(Widget,XtPointer,XVisibilityEvent *,
				                                  Boolean *);
static Boolean draw_grid_lines_work(XtPointer);
static void draw_grid_lines(int,int,int,int);
#if 0     /* xsky 1.0 grid code */
static int find_west_ra(int,int,int,int);
static int find_east_ra(int,int,int,int);
static int find_south_dec(int,int,int,int);
static int find_north_dec(int,int,int,int);
#endif     /* xsky 1.0 grid code */
static void draw_ra_line(int,int,int,int,int,int,int,int);
static void draw_dec_line(int,int,int,int,int,int,int,int);
static void draw_grid_dot(int,int,int,int,int,int);
static XtEventHandler cancel_find(Widget,XtPointer,XEvent *,Boolean *);
static XtCallbackProc FindObject(Widget,XtPointer,XtPointer);
static boolean find_object_by_name(char *);
static XtEventHandler HandleSkyButtonPress(Widget,XtPointer,
				              XButtonPressedEvent *,Boolean *);
static XtEventHandler HandleSkyButtonRelease(Widget,struct id_node *,
				             XButtonReleasedEvent *,Boolean *);
static void displayobjinfo(XButtonPressedEvent *);
static void identify_object(int,int);
static void remove_identification(int,int);
static void recenter_display(XButtonPressedEvent *);
static XtEventHandler handle_button2_motion(Widget,struct id_node *,
					             XMotionEvent *,Boolean *);
static void draw_text_box(int,int,int,int);
static void draw_id_texts(int,int,int,int);
static void add_id_to_list(struct id_node *);
static void remove_this_id(struct id_node *,struct id_node *);
static void update_center_position();
static void update_sky(struct display *);
static Boolean update_sky_work(XtPointer);
static void update_thumbs();
static void redraw_sky();
static XtCallbackProc display_zoom(Widget,XtPointer,XtPointer);
static XtCallbackProc inc_scroll(Widget,XtPointer,XtPointer);
static XtCallbackProc jump_scroll(Widget,XtPointer,XtPointer);
static int build_grid_intervals(int [],int);
static int int_compare(int *,int *);
static int build_dot_intervals(int [],int);
static struct obj_node *find_object(int,int,struct cat_header **);

/* external function prototypes */
extern void catalog_init();
extern boolean find_tok(char *,char *,char *);
extern void draw_id_text(Widget,char *,IDfont,int,int);
extern void X_to_pos_rad(struct display *,int,int,double *,double *);
extern void pos_rad_to_X(struct display *,double,double,int *,int *);
extern double ra_to_rad(int,int,int,int);
extern double dec_to_rad(int,int,int,int,int);
extern int ra_rad_to_thousandths(double);
extern int dec_rad_to_hundredths(double);
extern double ra_thousandths_to_rad(int);
extern double dec_hundredths_to_rad(int);
#ifdef ROUND_DISPLAY_POS
extern void round_ra(struct ra_pos *);
extern void round_dec(struct dec_pos *);
#endif
extern struct ra_pos rad_to_ra(double);
extern struct dec_pos rad_to_dec(double);
extern boolean calculate_position(struct display *,char *);
extern boolean ps_setup();
extern boolean ps_begin_header();
extern boolean ps_put_header(char *);
extern boolean ps_end_header();
extern boolean ps_begin_drawing(Dimension,Dimension);
extern boolean ps_draw_id(struct id_node *,Dimension);
extern boolean ps_end_drawing();
extern void ps_grid_newpath();
extern void ps_grid_endpath();
extern void ps_draw_point(double,double,Dimension);
extern void pos_rad_to_X_float(struct display *,double,double,float *,float *);
extern void draw_point(Widget,Position,Position);
extern void calculate_limits(struct display *);
extern boolean within_limits(struct display *,double,double);
#else   /* not __STDC__ */
/* static function prototypes */
static void printusage();
static void initialize_display();
static void buildmaindisplay();
static Widget makepanelbutton();
static Widget makepaneltoggle();
static void makecenterfield();
static void calculate_field();
static void setup_cursors();
static void setup_graphics();
static Boolean buildinfodisplay();
static Boolean buildcatalogmenu();
static Boolean buildfinddisplay();
static Boolean buildobjdisplay();
static Widget maketextfield();
static XtCallbackProc PrepareInfo();
static XtCallbackProc PrepareFind();
static Boolean make_big_initial();
static XtCallbackProc handle_grid_button();
static XtCallbackProc toggle_ids();
static XtCallbackProc erase_ids();
static XtCallbackProc popup_cat_menu();
static XtCallbackProc ps_draw_sky();
static XtCallbackProc Quit();
static void set_normal_cursor();
static void set_wait_cursor();
static void set_textbox_cursor();
static void draw_sky();
static boolean ps_write_file();
static XtEventHandler cancel_dpyinfo();
static XtCallbackProc DpyInfoAcked();
static XtCallbackProc DataAcked();
static XtEventHandler DataAckEvent();
static Boolean draw_grid_lines_work();
static void draw_grid_lines();
#if 0     /* xsky 1.0 grid code */
static int find_west_ra();
static int find_east_ra();
static int find_south_dec();
static int find_north_dec();
#endif     /* xsky 1.0 grid code */
static void draw_ra_line();
static void draw_dec_line();
static void draw_grid_dot();
static XtEventHandler cancel_find();
static XtCallbackProc FindObject();
static boolean find_object_by_name();
static XtEventHandler HandleSkyButtonPress();
static XtEventHandler HandleSkyButtonRelease();
static void displayobjinfo();
static void identify_object();
static void remove_identification();
static void recenter_display();
static XtEventHandler handle_button2_motion();
static void draw_text_box();
static void draw_id_texts();
static void add_id_to_list();
static void remove_this_id();
static void update_center_position();
static void update_sky();
static Boolean update_sky_work();
static void update_thumbs();
static void redraw_sky();
static XtCallbackProc display_zoom();
static XtCallbackProc inc_scroll();
static XtCallbackProc jump_scroll();
static int build_grid_intervals();
static int int_compare();
static int build_dot_intervals();
static struct obj_node *find_object();

/* external function prototypes */
extern void catalog_init();
extern boolean find_tok();
extern void draw_id_text();
extern void X_to_pos_rad();
extern void pos_rad_to_X();
extern double ra_to_rad();
extern double dec_to_rad();
extern int ra_rad_to_thousandths();
extern int dec_rad_to_hundredths();
extern double ra_thousandths_to_rad();
extern double dec_hundredths_to_rad();
#ifdef ROUND_DISPLAY_POS
extern void round_ra();
extern void round_dec();
#endif
extern struct ra_pos rad_to_ra();
extern struct dec_pos rad_to_dec();
extern boolean calculate_position();
extern boolean ps_setup();
extern boolean ps_begin_header();
extern boolean ps_put_header();
extern boolean ps_end_header();
extern boolean ps_begin_drawing();
extern boolean ps_draw_id();
extern boolean ps_end_drawing();
extern void ps_grid_newpath();
extern void ps_grid_endpath();
extern void ps_draw_point();
extern void pos_rad_to_X_float();
extern void draw_point();
extern void calculate_limits();
extern boolean within_limits();
#endif   /* __STDC__ */

/* magic option for "build binary databases only" */
#define MAGIC_OPTION   "-bin"

/* getopt() option-processing externals */
extern int optind, opterr;
extern char *optarg;

/* default initial display scale */
static float default_scale = INITIAL_PIXELS_PER_DEGREE;

/* default initial width and height */
static int default_width = INITIAL_WIDTH;
static int default_height = INITIAL_HEIGHT;

/* default initial magnitude limit */
static float default_maglim = INITIAL_MAGNITUDE;

/* note the space on the end for a fudge factor */
#define MAX_POS_STRING   "00h 00m 00s.000, -00d 00' 00\".00 "

/* colors for the spectral classes */
#define DEFAULT_O_COLOR   "Blue"
#define DEFAULT_B_COLOR   "DeepSkyBlue"
#define DEFAULT_A_COLOR   "White"
#define DEFAULT_F_COLOR   "NavajoWhite3"
#define DEFAULT_G_COLOR   "Yellow"
#define DEFAULT_K_COLOR   "Orange"
#define DEFAULT_M_COLOR   "Red"

/* fonts and buffer for object identification text */
#define LATIN_ID_FONTNAME   "helvetica10"
XFontStruct *latinidfontstr;
static char *latin_font = (char *)NULL;

#define GREEK_ID_FONTNAME   "symbol12"
XFontStruct *greekidfontstr;
static char *greek_font = (char *)NULL;

/* size of a Latin x for object drawing */
int x_height;
int x_width;

/* structure for retrieval of application resources */
typedef struct _instance_variables {
  String pos;
  float scale;
  int width;
  int height;
  float maglim;
  String latinfont;
  String greekfont;
  String ocolor, bcolor, acolor, fcolor, gcolor, kcolor, mcolor;
  Boolean autopopdown;
  String firstcat;
} instance_variable_rec;

static instance_variable_rec iv;

#define GETIVSTRING(resource_name,iv_element,default)                     \
  {                                                                       \
  resource_name,                                                          \
  XtCString,                                                              \
  XtRString, sizeof(String),                                              \
  XtOffsetOf(instance_variable_rec,iv_element),                           \
  XtRString, (XtPointer)default                                           \
  }

#define GETIVINT(resource_name,iv_element,default)                        \
  {                                                                       \
  resource_name,                                                          \
  XtCValue,                                                               \
  XtRInt, sizeof(int),                                                    \
  XtOffsetOf(instance_variable_rec,iv_element),                           \
  XtRInt, (XtPointer)default                                              \
  }

#define GETIVFLOAT(resource_name,iv_element,default)                      \
  {                                                                       \
  resource_name,                                                          \
  XtCValue,                                                               \
  XtRFloat, sizeof(float),                                                \
  XtOffsetOf(instance_variable_rec,iv_element),                           \
  XtRFloat, (XtPointer)default                                            \
  }

#define GETIVBOOLEAN(resource_name,iv_element,default)                    \
  {                                                                       \
  resource_name,                                                          \
  XtCValue,                                                               \
  XtRBoolean, sizeof(Boolean),                                            \
  XtOffsetOf(instance_variable_rec,iv_element),                           \
  XtRImmediate, (XtPointer)default                                        \
  }

static XtResource resources[] = {
  GETIVSTRING("displaypos",pos,INITIAL_POSITION),
  GETIVFLOAT("displayscale",scale,&default_scale),
  GETIVINT("displaywidth",width,&default_width),
  GETIVINT("displayheight",height,&default_height),
  GETIVFLOAT("displaymaglim",maglim,&default_maglim),
  GETIVSTRING("displaylatinfont",latinfont,LATIN_ID_FONTNAME),
  GETIVSTRING("displaygreekfont",greekfont,GREEK_ID_FONTNAME),
  GETIVSTRING("displaycolorO",ocolor,DEFAULT_O_COLOR),
  GETIVSTRING("displaycolorB",bcolor,DEFAULT_B_COLOR),
  GETIVSTRING("displaycolorA",acolor,DEFAULT_A_COLOR),
  GETIVSTRING("displaycolorF",fcolor,DEFAULT_F_COLOR),
  GETIVSTRING("displaycolorG",gcolor,DEFAULT_G_COLOR),
  GETIVSTRING("displaycolorK",kcolor,DEFAULT_K_COLOR),
  GETIVSTRING("displaycolorM",mcolor,DEFAULT_M_COLOR),
  GETIVBOOLEAN("autopopdown",autopopdown,True),
  GETIVSTRING("displayfirst",firstcat,INITIAL_CATALOG)
};

/* structure describing the current sky display state */
static struct display display;

/* position and size of toplevel widget before make-big call */
static Position old_x, old_y;
static Dimension old_width, old_height;
static boolean initial_values_captured = FALSE;

/* list of catalog header structures */
static struct cat_header *cat_list_head = (struct cat_header *)NULL;

/* initial list of active catalogs */
static char catlist[INITIAL_CATLIST_LEN + 1];

/* freelist pointer for displayable object list nodes */
struct obj_node *freelist = (struct obj_node *)NULL;

/* add an action to call the draw_sky routine */
#ifdef __STDC__
static XtActionProc sky_expose(Widget,XExposeEvent *,String *,Cardinal *);
#else
static XtActionProc sky_expose();
#endif
static XtActionsRec sky_actions[] = {
  {"sky_expose",(XtActionProc)sky_expose}
};

/* set up expose events to invoke the draw_sky action */
static String sky_trans = "<Expose>:  sky_expose()";

/* add an action to pop down the object selection menu */
#ifdef __STDC__
static XtActionProc popdown_cat_menu(Widget,XEvent *,String *,Cardinal *);
#else
static XtActionProc popdown_cat_menu();
#endif
static XtActionsRec obj_actions[] = {
  {"popdownObjMenu",(XtActionProc)popdown_cat_menu}
};

/* add an action to pop down the object finder display */
#ifdef __STDC__
static XtActionProc popdown_find_display(Widget,XEvent *,String *,Cardinal *);
#else
static XtActionProc popdown_find_display();
#endif
static XtActionsRec find_actions[] = {
  {"popdownFindDisplay",(XtActionProc)popdown_find_display}
};

/* add an action to find an object when <return> key is pressed */
#ifdef __STDC__
static XtActionProc find_object_in_text(Widget,XEvent *,String *,Cardinal *);
#else
static XtActionProc find_object_in_text();
#endif
static XtActionsRec find_text_actions[] = {
  {"findObject",(XtActionProc)find_object_in_text}
};

/* add an action to update the display center when <return> key is pressed */
#ifdef __STDC__
static XtActionProc display_center_edited(Widget,XEvent *,String *,Cardinal *);
#else
static XtActionProc display_center_edited();
#endif
static XtActionsRec center_text_actions[] = {
  {"updateCenter",(XtActionProc)display_center_edited}
};

/* add an action to update the display from edited Info display text */
#ifdef __STDC__
static XtActionProc DpyInfoEdited(Widget,XEvent *,String *,Cardinal *);
#else
static XtActionProc DpyInfoEdited();
#endif
static XtActionsRec info_text_actions[] = {
  {"updateInfo",(XtActionProc)DpyInfoEdited}
};

/* buffer to hold the information display */
static char info_display[350];

/* buffers to hold the editable display position and limiting magnitude */
static char pos_buffer[50], old_pos_buffer[50];
static char mag_buffer[10], old_mag_buffer[10];
static char scale_buffer[15], old_scale_buffer[15];

/* buffer to hold name of object to find */
#define MAXOBJNAME   50
static char object_buffer[MAXOBJNAME + 1];

/* buffer to hold the object information display */
char obj_info[2400];

static XrmValue blackstring = {
  sizeof("Black"),(caddr_t)"Black"
  };

static XrmValue whitestring = {
  sizeof("White"),(caddr_t)"White"
  };

int blackpixelval;

static XrmValue blackpixel = {
  sizeof(blackpixelval),(caddr_t)&blackpixelval
  };

int whitepixelval;

static XrmValue whitepixel = {
  sizeof(whitepixelval),(caddr_t)&whitepixelval
  };

/* declare some star colors and graphics contexts for each */
#define NUMSTARCOLORS   7
static char *starcolors[NUMSTARCOLORS] = {
  (char *)NULL,
  (char *)NULL,
  (char *)NULL,
  (char *)NULL,
  (char *)NULL,
  (char *)NULL,
  (char *)NULL
  };

static int colorpixelval[NUMSTARCOLORS];

GC star_gcs[NUMSTARCOLORS];

GC mono_star_gc;

GC obj_gc;

GC text_gc;

GC grid_gc;

/* list header for list of visual identifications */
static struct id_node *id_list = (struct id_node *)NULL;

/* user-specified magnitude limit */
static boolean opt_mag_limit_flag = FALSE;
static float opt_mag_limit;

/* user-specified display scale */
static boolean opt_display_scale_flag = FALSE;
static float opt_display_scale;

/* user-specified initial position */
static char *opt_pos = (char *)NULL;

/* user-specified initial catalog list */
static char *opt_catlist = (char *)NULL;

/* TRUE if color display */
boolean mono_flag = FALSE;

/* True if autopopdown of popup displays is enabled */
static Boolean autopopdown;

/* remember if user specified -L option */
static boolean big_flag = FALSE;

/* lowest magnitude to display */
float mag_limit = 8.0;

/* define a couple of cursors */
static Cursor normal_cursor;
static Cursor wait_cursor;
static Cursor invisible_cursor;

/* flag for display of grid lines */
static Boolean draw_grid = False;

/* flag for display of object IDs */
static Boolean draw_ids = True;

/* right ascension and declination line intervals (RA interval is in
 * thousandths of a second of time, dec interval is in hundredths of
 * a second of arc) */
static int ra_line_interval;
static int dec_line_interval;

/* table of allowed grid intervals (even divisors of one hour of time
 * or ten degrees of arc, expressed in thousandths of a second for
 * right ascension and hundredths of a degree for declination).  due
 * to a fortuitous happenstance, the number of thousandths of a second
 * of time in one hour is exactly the same as the number of hundredths
 * of a second of arc in ten degrees - so we only need one table! */
static int grid_intervals[300];
static int num_grid_intervals;
static boolean interval_table_built = FALSE;

/* PostScript output file */
extern FILE *psfile;

/* flag that we're drawing in PostScript, not on the display */
static boolean ps_draw_flag;

/* declare the widgets globally so callback routines can find them easily */
static Widget toplevel, form;
static Widget sky_widget, vert_scroll, horiz_scroll;
static Widget menuinfo, menugrid, menucat, menuhide, menuerase,
                                                   menufind, menups, menuquit;
static Widget centerbox, centerlabel, centertext;
static Widget pshell, infoform, infolabel, displaymag,
                                                    displayscale, ack, cancel;
static Widget catmenushell, catbox, catlabel;
static Widget zoomin, zoomout;
static Widget findshell, findbox, findlabel, findtext, findack, findcancel;



main(argc,argv)

int argc;
char *argv[];

{
  int c;
  int n;
  Arg sky_args[20];
  struct cat_header *catalog;

/* initialize and read the list of catalogs */
  catalog_init(&cat_list_head,catlist);

/* magic:  if the argument count is exactly two and the second argument is
 * exactly the magic option, just exit after having read the catalogs (and
 * thus built and saved the binary dumps); avoid all contact with X */
  if ((argc == 2) && (strcmp(argv[1],MAGIC_OPTION) == EQUAL))
    exit(0);

/* initialize the Intrinsics, creating top-level shell widget */
  toplevel = XtInitialize(
			  argv[0],
			  "XSky",
			  (XrmOptionDescRec *)NULL,
			  (Cardinal)0,
			  &argc,
			  argv);
#ifdef DEBUG
  printf("XtInitialize returned\n");
#endif

/* check for any options of our own */
  while ((c = getopt(argc,argv,"hLbm:f:g:c:s:p:O:B:A:F:G:K:M:")) != EOF)
    switch ((char)c) {
    case 'L':     /* make the display fill the screen */
      big_flag = TRUE;
      break;
    case 'b':     /* black-and-white even if color display */
      mono_flag = TRUE;
      break;
    case 'c':     /* initial catalog list */
      opt_catlist = optarg;
      break;
    case 'f':     /* font for object identifications */
      latin_font = optarg;
      break;
    case 'g':     /* Greek font for object identifications */
      greek_font = optarg;
      break;
    case 'h':
      printusage(argv);
      break;
    case 'p':     /* initial display position */
      opt_pos = optarg;
      break;
    case 'm':     /* set limiting lower magnitude */
      if (optarg == (char *)NULL)
	printusage(argv);
      else {
	opt_mag_limit = atof(optarg);
	opt_mag_limit_flag = TRUE;
      }
      break;
    case 's':     /* set display scale */
      if (optarg == (char *)NULL)
	printusage(argv);
      else {
	opt_display_scale = atof(optarg);
	opt_display_scale_flag = TRUE;
      }
      break;
    case 'O':
      starcolors[O] = optarg;
      break;
    case 'B':
      starcolors[B] = optarg;
      break;
    case 'A':
      starcolors[A] = optarg;
      break;
    case 'F':
      starcolors[F] = optarg;
      break;
    case 'G':
      starcolors[G] = optarg;
      break;
    case 'K':
      starcolors[K] = optarg;
      break;
    case 'M':
      starcolors[M] = optarg;
      break;
    default:
      printusage(argv);
      break;
    }

/* register the sky actions */
  XtAddActions(sky_actions,XtNumber(sky_actions));

/* register the object menu actions */
  XtAddActions(obj_actions,XtNumber(obj_actions));

/* register the object finder display actions */
  XtAddActions(find_actions,XtNumber(find_actions));
  XtAddActions(find_text_actions,XtNumber(find_text_actions));

/* register the display center text actions */
  XtAddActions(center_text_actions,XtNumber(center_text_actions));

/* register the Info text actions */
  XtAddActions(info_text_actions,XtNumber(info_text_actions));

/* fetch the application-specific resources */
  XtGetApplicationResources(toplevel,(XtPointer)&iv,resources,
			                         XtNumber(resources),
			                         (ArgList)NULL,(Cardinal)0);

/* initialize the display structure to the startup position and size */
  initialize_display();

/* build the main display panel */
  buildmaindisplay();

/* get the values of black and white */
  if (XtConvertAndStore(sky_widget,XtRString,&blackstring,
			                            XtRPixel,&blackpixel))
    ;
  else
    printf("XtConvertAndStore black failed\n");
  if (XtConvertAndStore(sky_widget,XtRString,&whitestring,
			                            XtRPixel,&whitepixel))
    ;
  else
    printf("XtConvertAndStore white failed\n");

/* set the show flag for each catalog in the initial display list */
  catalog = cat_list_head;
  while (catalog != (struct cat_header *)NULL) {
    catalog->show_flag = find_tok(catlist,catalog->menu_name,",");
    catalog = catalog->next_cat;
  }

/* build the lists of displayable objects */
  calculate_field(&display);
#ifdef DEBUG
  printf("catalogs read and initial field calculated\n");
#endif

/* realize the widgets */
   XtRealizeWidget(toplevel);
#ifdef DEBUG
  printf("XtRealizeWidget returned\n");
#endif

/* make the sky display's background black, border and foreground white */
  n = 0;
  XtSetArg(sky_args[n],XtNforeground,whitepixelval);  n++;
  XtSetArg(sky_args[n],XtNbackground,blackpixelval);  n++;
  XtSetArg(sky_args[n],XtNborderColor,whitepixelval); n++;
  XtSetValues(sky_widget,sky_args,n);

/* set up the cursors we want */
  setup_cursors();

/* create the graphics contexts */
  setup_graphics();

/* build the popup shells for the object information */
  catalog = cat_list_head;
  while (catalog != (struct cat_header *)NULL) {
#ifdef WORKPROC
    (void)XtAddWorkProc(buildobjdisplay,(XtPointer)catalog);
#else
    (void)buildobjdisplay(catalog);
#endif
    catalog = catalog->next_cat;
  }

/* build the object finder display */
#ifdef WORKPROC
  (void)XtAddWorkProc(buildfinddisplay,(XtPointer)NULL);
#else
  (void)buildfinddisplay();
#endif

/* build the object selection popup shell */
#ifdef WORKPROC
  (void)XtAddWorkProc(buildcatalogmenu,(XtPointer)NULL);
#else
  (void)buildcatalogmenu();
#endif

/* start a work procedure to build the popup shell for the info display */
#ifdef WORKPROC
  (void)XtAddWorkProc(buildinfodisplay,(XtPointer)NULL);
#else
  (void)buildinfodisplay();
#endif

/* if the command line calls for a large display, do it */
  if (big_flag)
#ifdef WORKPROC
    (void)XtAddWorkProc(make_big_initial,(XtPointer)NULL);
#else
     ;    /* this doesn't work this way without a WorkProc */
#endif

#ifdef DEBUG
  printf("entering main loop\n");
#endif
/* enter the event loop */
  XtMainLoop();

/* NOTREACHED */
}



/* set up the initial position of the display */

static void initialize_display()

{
/* save the display autopopdown flag */
  autopopdown = iv.autopopdown;

/* save the initial display position */
  strcpy(pos_buffer,(char *)iv.pos);
  /* override the initial display position with the command-line value */
  if (opt_pos != (char *)NULL)
    strcpy(pos_buffer,opt_pos);
#ifdef DEBUG
  printf("initial position buffer = %s\n",pos_buffer);
#endif
  if (! calculate_position(&display,pos_buffer)) {
    /* supplied position is bad; use a known good initial position */
    strcpy(pos_buffer,INITIAL_POSITION);
    (void)calculate_position(&display,pos_buffer);
    /* and ring a bell to signify our displeasure */
    XBell(XtDisplay(sky_widget),0);
  }
#ifdef DEBUG
  printf("display center = %lf, %lf radians\n",display.center_ra_rad,
	                                             display.center_dec_rad);
#endif

/* save the initial active catalog list */
  strcpy(catlist,(char *)iv.firstcat);
  /* override the initial display position with the command-line value */
  if (opt_catlist != (char *)NULL)
    strcpy(catlist,opt_catlist);
#ifdef DEBUG
  printf("initial catalog list = %s\n",catlist);
#endif

/* remember the initial display position forever for the scrollbars */
  display.initial_ra_rad = display.center_ra_rad;
  display.initial_dec_rad = display.center_dec_rad;

/* save the initial display scale (display scale is degrees per pixel) */
  display.scale = 1.0 / iv.scale;
  /* override the display scale with the command-line value */
  if (opt_display_scale_flag)
    display.scale = 1.0 / opt_display_scale;
#ifdef DEBUG
  printf("initial display scale = %f degrees per pixel",display.scale);
  printf(" or %f pixels per degree\n",1.0 / display.scale);
#endif
/* set the initial scale factor string */
  sprintf(scale_buffer,"%.1f",1.0 / display.scale);

/* set the initial width and height */
  display.width = iv.width;
  display.height = iv.height;

#ifdef DEBUG
  printf("display dimensions %d x %d pixels\n",display.width,display.height);
#endif

/* compute the new display width and height in radians */
  display.width_rad = (display.width * display.scale) / DEGREES_PER_RAD;
  display.height_rad = (display.height * display.scale) / DEGREES_PER_RAD;
#ifdef DEBUG
  printf("display dimensions = %lf, %lf radians\n",display.width_rad,
	                                                 display.height_rad);
#endif

/* set the initial limiting magnitude */
  mag_limit = iv.maglim;
  /* override the limiting magnitude with the command-line value */
  if (opt_mag_limit_flag)
    mag_limit = opt_mag_limit;
  sprintf(mag_buffer,"%.1f\n",mag_limit);

/* set the spectral class colors for the display */
  if (starcolors[O] == (char *)NULL)
    starcolors[O] = (char *)iv.ocolor;
  if (starcolors[B] == (char *)NULL)
    starcolors[B] = (char *)iv.bcolor;
  if (starcolors[A] == (char *)NULL)
    starcolors[A] = (char *)iv.acolor;
  if (starcolors[F] == (char *)NULL)
    starcolors[F] = (char *)iv.fcolor;
  if (starcolors[G] == (char *)NULL)
    starcolors[G] = (char *)iv.gcolor;
  if (starcolors[K] == (char *)NULL)
    starcolors[K] = (char *)iv.kcolor;
  if (starcolors[M] == (char *)NULL)
    starcolors[M] = (char *)iv.mcolor;

  return;
}



/* build the main display panel for the application */

static void buildmaindisplay()

{
  int n;
  Arg widget_args[10];
  int button_num;
  Widget longest;
  Dimension longlength;

/* create a form widget to act as a container */
  form = XtCreateManagedWidget(
			      "form",          /* widget name */
			      formWidgetClass, /* widget class */
			      toplevel,        /* parent widget */
			      (Arg *)NULL,     /* argument list */
			      0);              /* argument list size */

/* add the info button */
  button_num = 0;
  longlength = 0;
  menuinfo = makepanelbutton(button_num++,(Widget)NULL,form,
			   (XtCallbackProc)PrepareInfo,(XtPointer)NULL,
			                                &longest,&longlength);

/* add the catalog menu button */
  menucat = makepanelbutton(button_num++,menuinfo,form,
			   (XtCallbackProc)popup_cat_menu,(XtPointer)NULL,
			                                &longest,&longlength);

/* add the zoom buttons */
  zoomin = makepanelbutton(button_num++,menucat,form,
			   (XtCallbackProc)display_zoom,(XtPointer)1,
			                                &longest,&longlength);
  zoomout = makepanelbutton(button_num++,zoomin,form,
			   (XtCallbackProc)display_zoom,(XtPointer)-1,
			                                &longest,&longlength);

/* add the grid button */
  menugrid = makepaneltoggle(button_num++,zoomout,form,
			   (XtCallbackProc)handle_grid_button,(XtPointer)NULL,
			                                &longest,&longlength);

/* add buttons to hide and erase IDs */
  menuhide = makepaneltoggle(button_num++,menugrid,form,
			   (XtCallbackProc)toggle_ids,(XtPointer)NULL,
			                                &longest,&longlength);

  menuerase = makepanelbutton(button_num++,menuhide,form,
			   (XtCallbackProc)erase_ids,(XtPointer)NULL,
			                                &longest,&longlength);

/* add the find button */
  menufind = makepanelbutton(button_num++,menuerase,form,
			   (XtCallbackProc)PrepareFind,(XtPointer)NULL,
			                                &longest,&longlength);

/* add the print button */
  menups = makepanelbutton(button_num++,menufind,form,
			   (XtCallbackProc)ps_draw_sky,(XtPointer)NULL,
			                                &longest,&longlength);

/* add the quit button */
  menuquit = makepanelbutton(button_num++,menups,form,
			   (XtCallbackProc)Quit,(XtPointer)NULL,
                                                        &longest,&longlength);

/* add the text field for the display center and get it updated */
  makecenterfield(form,longest);
  update_center_position();

/* build the argument list for the sky widget creation */
  n = 0;
  XtSetArg(widget_args[n],XtNwidth,display.width);                        n++;
  XtSetArg(widget_args[n],XtNheight,display.height);                      n++;
  XtSetArg(widget_args[n],XtNtranslations,
	                            XtParseTranslationTable(sky_trans));  n++;
  XtSetArg(widget_args[n],XtNfromHoriz,longest);                          n++;
  XtSetArg(widget_args[n],XtNhorizDistance,5);                            n++;
  XtSetArg(widget_args[n],XtNfromVert,centerbox);                         n++;
  XtSetArg(widget_args[n],XtNvertDistance,5);                             n++;
/* create the core widget */
  sky_widget = XtCreateManagedWidget("sky",
				     coreWidgetClass,
				     form,
				     widget_args,n);

/* add an event handler to field pointer button presses in the sky widget */
  XtAddEventHandler(sky_widget,ButtonPressMask | OwnerGrabButtonMask,False,
		        (XtEventHandler)HandleSkyButtonPress,(XtPointer)NULL);

/* add vertical and horizontal scrollbars to move the sky display */
  n = 0;
  XtSetArg(widget_args[n],XtNlength,display.height);                      n++;
  XtSetArg(widget_args[n],XtNorientation,XtorientVertical);               n++;
  XtSetArg(widget_args[n],XtNfromHoriz,sky_widget);                       n++;
  XtSetArg(widget_args[n],XtNhorizDistance,0);                            n++;
  XtSetArg(widget_args[n],XtNfromVert,centerbox);                         n++;
  XtSetArg(widget_args[n],XtNvertDistance,5);                             n++;
  vert_scroll = XtCreateManagedWidget("scroll",
				      scrollbarWidgetClass,
				      form,
				      widget_args,n);

  /* add callbacks for incremental and jump scrolling */
  XtAddCallback(vert_scroll,XtNscrollProc,(XtCallbackProc)inc_scroll,
		                                             (XtPointer)NULL);
  XtAddCallback(vert_scroll,XtNjumpProc,(XtCallbackProc)jump_scroll,
		                                             (XtPointer)NULL);

  n = 0;
  XtSetArg(widget_args[n],XtNlength,display.width);                       n++;
  XtSetArg(widget_args[n],XtNorientation,XtorientHorizontal);             n++;
  XtSetArg(widget_args[n],XtNfromVert,sky_widget);                        n++;
  XtSetArg(widget_args[n],XtNvertDistance,0);                             n++;
  XtSetArg(widget_args[n],XtNfromHoriz,longest);                          n++;
  XtSetArg(widget_args[n],XtNhorizDistance,5);                            n++;
  horiz_scroll = XtCreateManagedWidget("scroll",
				       scrollbarWidgetClass,
				       form,
				       widget_args,n);

  /* add a callback for incremental scrolling */
  XtAddCallback(horiz_scroll,XtNscrollProc,(XtCallbackProc)inc_scroll,
		                                             (XtPointer)NULL);
  XtAddCallback(horiz_scroll,XtNjumpProc,(XtCallbackProc)jump_scroll,
		                                             (XtPointer)NULL);

/* set up the thumb positions and sizes */
  update_thumbs();

  return;
}



/* make a panel button for the main display */

static Widget makepanelbutton(i,prev_widget,parent,callback,param,
			                                   longest,longlength)

int i;
Widget prev_widget;
Widget parent;
XtCallbackProc callback;
XtPointer param;
Widget *longest;
Dimension *longlength;

{
  char buff[15];
  int n;
  Arg widget_args[5];
  Widget w;
  Dimension length;

/* create the panel button name */
  sprintf(buff,"panelbutton%d",i);

/* add the info button */
  n = 0;
  XtSetArg(widget_args[n],XtNjustify,XtJustifyLeft);                      n++;
  if (prev_widget != (Widget)NULL) {
    XtSetArg(widget_args[n],XtNfromVert,prev_widget);                     n++;
    XtSetArg(widget_args[n],XtNvertDistance,5);                           n++;
  }
  w = XtCreateManagedWidget(buff,commandWidgetClass,parent,widget_args,n);

  /* add the callback routine, if any */
  if (callback != (XtCallbackProc)NULL)
    XtAddCallback(w,XtNcallback,(XtCallbackProc)callback,param);

  /* get the button width and update the longest button */
  n = 0;
  XtSetArg(widget_args[n],XtNwidth,&length);                              n++;
  XtGetValues(w,widget_args,n);

  if (length > *longlength) {
    *longlength = length;
    *longest = w;
  }

  return(w);
}



/* make a panel toggle for the main display */

static Widget makepaneltoggle(i,prev_widget,parent,callback,param,
			                                   longest,longlength)

int i;
Widget prev_widget;
Widget parent;
XtCallbackProc callback;
XtPointer param;
Widget *longest;
Dimension *longlength;

{
  char buff[15];
  int n;
  Arg widget_args[5];
  Widget w;
  Dimension length;

/* create the panel button name */
  sprintf(buff,"panelbutton%d",i);

/* add the info button */
  n = 0;
  XtSetArg(widget_args[n],XtNjustify,XtJustifyLeft);                      n++;
  if (prev_widget != (Widget)NULL) {
    XtSetArg(widget_args[n],XtNfromVert,prev_widget);                     n++;
    XtSetArg(widget_args[n],XtNvertDistance,5);                           n++;
  }
  w = XtCreateManagedWidget(buff,toggleWidgetClass,parent,widget_args,n);

  /* add the callback routine, if any */
  if (callback != NULLVOIDFUNCTION)
    XtAddCallback(w,XtNcallback,(XtCallbackProc)callback,param);

  /* get the button width and update the longest button */
  n = 0;
  XtSetArg(widget_args[n],XtNwidth,&length);                              n++;
  XtGetValues(w,widget_args,n);

  if (length > *longlength) {
    *longlength = length;
    *longest = w;
  }

  return(w);
}



/* create a labelled text field for the display center position */

static void makecenterfield(parent,leftwidget)

Widget parent;
Widget leftwidget;

{
  int n;
  Arg widgetargs[10];
  Pixel background;
  XFontStruct *font;
  int dummy;
  XCharStruct overall_size;

/* create a box widget to act as a container */
  n = 0;
  XtSetArg(widgetargs[n],XtNfromHoriz,leftwidget);                       n++;
  XtSetArg(widgetargs[n],XtNhorizDistance,5);                            n++;
  XtSetArg(widgetargs[n],XtNorientation,XtorientHorizontal);             n++;
  centerbox = XtCreateManagedWidget("centerbox",boxWidgetClass,parent,
				                        (Arg *)widgetargs,n);

/* get the background color of the form, and use it for invisible borders */
  n = 0;
  XtSetArg(widgetargs[n],XtNbackground,&background);                     n++;
  XtGetValues(centerbox,widgetargs,n);

/* create a label widget to identify the text field */
  n = 0;
  XtSetArg(widgetargs[n],XtNlabel,"Display center");                     n++;
  XtSetArg(widgetargs[n],XtNborderColor,background);                     n++;
  centerlabel = XtCreateManagedWidget("centerlabel",labelWidgetClass,
				             centerbox,(Arg *)widgetargs,n);

/* get the font the label uses, and use it for the text widgets */
  n = 0;
  XtSetArg(widgetargs[n],XtNfont,&font);                                 n++;
  XtGetValues(centerlabel,widgetargs,n);

/* figure out the correct length for the text widgets */
  XTextExtents(font,MAX_POS_STRING,strlen(MAX_POS_STRING),
	                                 &dummy,&dummy,&dummy,&overall_size);

/* create the text widget to hold the editable field value */
  n = 0;
  XtSetArg(widgetargs[n],XtNfont,font);                                  n++;
  XtSetArg(widgetargs[n],XtNuseStringInPlace,True);                      n++;
  XtSetArg(widgetargs[n],XtNeditType,XawtextEdit);                       n++;
  XtSetArg(widgetargs[n],XtNfromHoriz,centerlabel);                      n++;
  XtSetArg(widgetargs[n],XtNhorizDistance,5);                            n++;
  XtSetArg(widgetargs[n],XtNwidth,overall_size.width);                   n++;
  XtSetArg(widgetargs[n],XtNlength,sizeof(MAX_POS_STRING));              n++;
  XtSetArg(widgetargs[n],XtNstring,pos_buffer);                          n++;
  centertext = XtCreateManagedWidget("centertext",asciiTextWidgetClass,
				              centerbox,(Arg *)widgetargs,n);

/* return the text widget as the function value */
  return;
}



/* create popup catalog submenu for sky program */

static Boolean buildcatalogmenu()

{
  int n;
  Arg menuargs[5];
  Pixel background;
  int button_num;
  struct cat_header *catalog;
  char namebuff[20];

/* create a popup shell to hold the menu */
  catmenushell = XtCreatePopupShell("catmenu",transientShellWidgetClass,
				                      toplevel,(Arg *)NULL,0);

/* put a box in the transient shell */
  catbox = XtCreateManagedWidget("catbox",boxWidgetClass,catmenushell,
				                               (Arg *)NULL,0);

/* get the background color of the box, and use it for invisible borders */
  n = 0;
  XtSetArg(menuargs[n],XtNbackground,&background);                        n++;
  XtGetValues(catbox,menuargs,n);

/* put a label in the box */
  n = 0;
  XtSetArg(menuargs[n],XtNborderColor,background);                        n++;
  catlabel = XtCreateManagedWidget("catlabel",labelWidgetClass,catbox,
				                           (Arg *)menuargs,n);

/* initialize the button number */
  button_num = 0;

/* create a selection button for each available catalog */
  catalog = cat_list_head;
  while (catalog != (struct cat_header *)NULL) {
    /* create the button name */
    sprintf(namebuff,"catbutton%d",button_num++);

    /* create the menu item widget with the given label */
    n = 0;
    XtSetArg(menuargs[n],XtNlabel,catalog->menu_name);                    n++;
    catalog->catselect = XtCreateManagedWidget(namebuff,toggleWidgetClass,
					                   catbox,menuargs,n);

    /* set the initial state according to the flag value */
    n = 0;
    if (catalog->show_flag) {
      XtSetArg(menuargs[n],XtNstate,True);                                n++;
    }
    else {
      XtSetArg(menuargs[n],XtNstate,False);                               n++;
    }
    XtSetValues(catalog->catselect,menuargs,n);

    /* step to the next catalog */
    catalog = catalog->next_cat;
  }

  return(True);
}



/* create a popup object selection display for the sky program */

static Boolean buildfinddisplay()

{
  int n;
  Arg widgetargs[10];
  Pixel background;

/* create a popup shell to hold the display widgets */
  findshell = XtCreatePopupShell("findshell",transientShellWidgetClass,
			                             toplevel,(Arg *)NULL,0);

/* create a box widget to act as a container */
  findbox = XtCreateManagedWidget("findbox",boxWidgetClass,findshell,
				                              (Arg *)NULL,0);

/* get the background color of the box, and use it for invisible borders */
  n = 0;
  XtSetArg(widgetargs[n],XtNbackground,&background);                     n++;
  XtGetValues(findbox,widgetargs,n);

/* put a label in the box */
  n = 0;
  XtSetArg(widgetargs[n],XtNborderColor,background);                     n++;
  findlabel = XtCreateManagedWidget("findlabel",labelWidgetClass,findbox,
				                        (Arg *)widgetargs,n);

/* put in a text widget to hold the editable information */
  n = 0;
  XtSetArg(widgetargs[n],XtNuseStringInPlace,True);                      n++;
  XtSetArg(widgetargs[n],XtNeditType,XawtextEdit);                       n++;
  XtSetArg(widgetargs[n],XtNfromVert,findlabel);                         n++;
  XtSetArg(widgetargs[n],XtNvertDistance,5);                             n++;
  XtSetArg(widgetargs[n],XtNwidth,200);                                  n++;
#ifndef NOTEXT
  XtSetArg(widgetargs[n],XtNlength,MAXOBJNAME + 1);                      n++;
  XtSetArg(widgetargs[n],XtNstring,object_buffer);                       n++;
#endif
  findtext = XtCreateManagedWidget("findtext",asciiTextWidgetClass,findbox,
				                               widgetargs,n);
#ifdef NOTEXT
  XtVaSetValues(findtext,XtNlength,50,(String *)NULL);
#endif

/* initialize the object selection buffer */
  object_buffer[0] = '\0';

/* add a command widget to allow the user to tell us he's done playing */
  findack = XtCreateManagedWidget("findack",commandWidgetClass,findbox,
				                               (Arg *)NULL,0);

/* add a command widget to allow the user to cancel alterations */
  findcancel = XtCreateManagedWidget("cancel",commandWidgetClass,findbox,
				                               (Arg *)NULL,0);

/* add an event handler to detect when the pointer leaves the window */
  XtAddEventHandler(findshell,LeaveWindowMask,False,
		                 (XtEventHandler)cancel_find,(XtPointer)NULL);

/* add the callbacks necessary for the command buttons */

  /* what do to when the user hits the "acknowledge" button */
  XtAddCallback(findack,XtNcallback,(XtCallbackProc)FindObject,
		                                             (XtPointer)NULL);

  /* what do to when the user hits the "cancel" button */
  XtAddCallback(findcancel,XtNcallback,(XtCallbackProc)FindObject,
		                                               (XtPointer)-1);

  return(True);
}



/* build the popup display for the info button */

static Boolean buildinfodisplay()

{
  int n;
  Arg widgetargs[20];
  XFontStruct *labelfontstr;
  Pixel background;
  Widget formwidget;
  XCharStruct overall_size;
  int dummy;

/* create a popup shell to hold the display widgets */
  pshell = XtCreatePopupShell("displaydata",transientShellWidgetClass,
			                             toplevel,(Arg *)NULL,0);

/* create a form widget to act as a container */
  infoform = XtCreateManagedWidget("form",formWidgetClass,pshell,
				                              (Arg *)NULL,0);

/* get the background color of the form, and use it for invisible borders */
  n = 0;
  XtSetArg(widgetargs[n],XtNbackground,&background);                     n++;
  XtGetValues(infoform,widgetargs,n);

/* put in a label widget to display the fixed information */
  n = 0;
  XtSetArg(widgetargs[n],XtNborderColor,background);                     n++;
  infolabel = XtCreateManagedWidget("label",labelWidgetClass,infoform,
				                        (Arg *)widgetargs,n);

/* get the font the label uses, and use it for the text widgets */
  n = 0;
  XtSetArg(widgetargs[n],XtNfont,&labelfontstr);                         n++;
  XtGetValues(infolabel,widgetargs,n);

/* while we're doing text stuff, get and save the width and height of an x */
  XTextExtents(labelfontstr,"x",1,&dummy,&dummy,&dummy,&overall_size);
  x_height = overall_size.ascent;
  x_width = overall_size.width;

/* create the editable text fields for the display parameters */

  /* start with no left widget */
  formwidget = (Widget)NULL;

#if 0
  /* create the panel for the display center field */
  displaypos = maketextfield(infoform,&formwidget,labelfontstr,background,
			     "centered at",pos_buffer,sizeof(MAX_POS_STRING));
#endif

  /* create the panel for the magnitude limit field */
  displaymag = maketextfield(infoform,&formwidget,labelfontstr,background,
			          "mag. limit",mag_buffer,sizeof(mag_buffer));

  /* create the panel for the display scale field */
  displayscale = maketextfield(infoform,&formwidget,labelfontstr,background,
			 "scale (pix/deg)",scale_buffer,sizeof(scale_buffer));

/* add a command widget to allow the user to tell us he's done playing */
  n = 0;
  XtSetArg(widgetargs[n],XtNfromVert,formwidget);                         n++;
  XtSetArg(widgetargs[n],XtNvertDistance,5);                              n++;
  ack = XtCreateManagedWidget("ack",commandWidgetClass,infoform,widgetargs,n);

/* add a command widget to allow the user to cancel alterations */
  XtSetArg(widgetargs[n],XtNfromHoriz,ack);                               n++;
  XtSetArg(widgetargs[n],XtNhorizDistance,5);                             n++;
  cancel = XtCreateManagedWidget("cancel",commandWidgetClass,infoform,
			                                        widgetargs,n);

/* add an event handler to detect when the pointer leaves the window */
  if (autopopdown)
    XtAddEventHandler(pshell,LeaveWindowMask,False,
		              (XtEventHandler)cancel_dpyinfo,(XtPointer)NULL);

/* add the callbacks necessary for the command buttons */

  /* what do to when the user hits the "acknowledge" button */
  XtAddCallback(ack,XtNcallback,(XtCallbackProc)DpyInfoAcked,(XtPointer)NULL);

  /* what do to when the user hits the "cancel" button */
  XtAddCallback(cancel,XtNcallback,(XtCallbackProc)DpyInfoAcked,(XtPointer)-1);

  return(True);
}



/* create one labelled text field for the info display */

static Widget maketextfield(parent,leftwidget,font,bordercolor,
			                                label,buffer,buflen)

Widget parent;
Widget *leftwidget;
XFontStruct *font;
Pixel bordercolor;
char *label;
char *buffer;
int buflen;

{
  int n;
  Arg widgetargs[10];
  Widget paramform;
  Widget paramlabel;
  int dummy;
  XCharStruct overall_size;
  Widget paramtext;

/* create a form widget to act as a container */
  n = 0;
  XtSetArg(widgetargs[n],XtNfromVert,infolabel);                         n++;
  XtSetArg(widgetargs[n],XtNvertDistance,5);                             n++;
  if (*leftwidget != (Widget)NULL) {
    XtSetArg(widgetargs[n],XtNfromHoriz,*leftwidget);                    n++;
    XtSetArg(widgetargs[n],XtNhorizDistance,10);                         n++;
  }
  paramform = XtCreateManagedWidget("paramform",formWidgetClass,parent,
				                        (Arg *)widgetargs,n);

/* create a label widget to identify the text field */
  n = 0;
  XtSetArg(widgetargs[n],XtNfont,font);                                  n++;
  XtSetArg(widgetargs[n],XtNlabel,label);                                n++;
  XtSetArg(widgetargs[n],XtNborderColor,bordercolor);                    n++;
  paramlabel = XtCreateManagedWidget("paramlabel",labelWidgetClass,
				              paramform,(Arg *)widgetargs,n);

/* figure out the length of one space for the text widget */
  XTextExtents(font," ",1,&dummy,&dummy,&dummy,&overall_size);

/* create the text widget to hold the editable field value */
  n = 0;
  XtSetArg(widgetargs[n],XtNfont,font);                                  n++;
  XtSetArg(widgetargs[n],XtNuseStringInPlace,True);                      n++;
  XtSetArg(widgetargs[n],XtNeditType,XawtextEdit);                       n++;
  XtSetArg(widgetargs[n],XtNfromVert,paramlabel);                        n++;
  XtSetArg(widgetargs[n],XtNvertDistance,5);                             n++;
  XtSetArg(widgetargs[n],XtNwidth,overall_size.width * buflen);          n++;
#ifndef NOTEXT
  XtSetArg(widgetargs[n],XtNstring,buffer);                              n++;
  XtSetArg(widgetargs[n],XtNlength,buflen);                              n++;
#endif
  paramtext = XtCreateManagedWidget("paramtext",asciiTextWidgetClass,
				              paramform,(Arg *)widgetargs,n);
#ifdef NOTEXT
  XtVaSetValues(paramtext,XtNlength,buflen,(String *)NULL);
#endif

/* return the containing form widget */
  *leftwidget = paramform;

/* return the text widget as the function value */
  return(paramtext);
}



/* build a popup display for the object info information */

static Boolean buildobjdisplay(catptr)

XtPointer catptr;

{
  int n;
  Arg widgetargs[5];
  struct cat_header *catalog = (struct cat_header *)catptr;

/* create the popup shell widget to hold the display */
  n = 0;
  XtSetArg(widgetargs[n],XtNallowShellResize,True);                      n++;
  catalog->datashell = XtCreatePopupShell("datashell",
					           transientShellWidgetClass,
			                              toplevel,widgetargs,n);

/* put a dialog widget in the popup shell to display the data */
  n = 0;
  XtSetArg(widgetargs[n],XtNresize,True);                                n++;
  catalog->datadialog = XtCreateManagedWidget("datadialog",dialogWidgetClass,
				              catalog->datashell,widgetargs,n);

/* add a command widget to allow the user to tell us he's done reading */
  catalog->dataack = XtCreateManagedWidget("ack",commandWidgetClass,
			                    catalog->datadialog,(Arg *)NULL,0);

/* what do to when the user hits the "ack" button */
  XtAddCallback(catalog->dataack,XtNcallback,(XtCallbackProc)DataAcked,
		                                (XtPointer)catalog->datashell);

/* add an event handler to fake an ack when the window gets obscured */
  if (autopopdown)
    XtAddEventHandler(catalog->datashell,VisibilityChangeMask,False,
		                   (XtEventHandler)DataAckEvent,(XtPointer)-1);

  return(True);
}



/* set up the cursors */

static void setup_cursors()

{
  Display *sky_display;
  Window sky_window;
  Pixmap cursor_pixmap;
  XtGCMask gcmask;
  XGCValues gcvalues;
  GC zero_gc;
  XColor blackcolor;

/* get the display and the screen */
  sky_display = XtDisplay(sky_widget);
  sky_window = XtWindow(sky_widget);

/* create the cursors we will use */
  normal_cursor = XCreateFontCursor(sky_display,XC_trek);
  wait_cursor = XCreateFontCursor(sky_display,XC_watch);

/* create an invisible cursor, too (boy, is this hard!) */

  /* create a 1 x 1 pixmap, deep enough for the screen */
  cursor_pixmap = XCreatePixmap(sky_display,(Drawable)sky_window,1,1,1);

  /* create a graphics context that will draw zeroes */
  gcmask = GCFunction;
  gcvalues.function = GXclear;
  zero_gc = XCreateGC(sky_display,(Drawable)cursor_pixmap,gcmask,&gcvalues);

  /* draw a zero into the cursor pixmap */
  XDrawPoint(sky_display,(Drawable)cursor_pixmap,zero_gc,0,0);

  /* set up an XColor with arbitrary pixel values */
  blackcolor.red = 0;
  blackcolor.green = 0;
  blackcolor.blue = 0;

  /* make a cursor out of this pixmap */
  invisible_cursor = XCreatePixmapCursor(sky_display,
					         cursor_pixmap,cursor_pixmap,
					         &blackcolor,&blackcolor,0,0);

/* start with the normal cursor */
  set_normal_cursor();

/* free up what we can */
  XFreeGC(sky_display,zero_gc);

  return;
}



/* reset the normal cursor in the sky widget */

static void set_normal_cursor()

{
/* set the normal cursor in the sky widget window */
  XDefineCursor(XtDisplay(sky_widget),XtWindow(sky_widget),normal_cursor);

  return;
}



/* set the "waiting" cursor in the sky widget */

static void set_wait_cursor()

{
/* set the wait cursor in the sky widget window */
  XDefineCursor(XtDisplay(sky_widget),XtWindow(sky_widget),wait_cursor);

  return;
}



/* set the "text-box" cursor in the sky widget */

static void set_textbox_cursor()

{
/* set the wait cursor in the sky widget window */
  XDefineCursor(XtDisplay(sky_widget),XtWindow(sky_widget),invisible_cursor);

  return;
}



/* set up the graphics contexts needed to draw and identify objects */

static void setup_graphics()

{
  XWindowAttributes attr;
  int i;
  Colormap colormap;
  XColor rgbdef;
  XtGCMask gcmask;
  XGCValues gcvalues;

/* get attributes of the sky window */
  XGetWindowAttributes(XtDisplay(sky_widget),XtWindow(sky_widget),&attr);
  if (((attr.visual)->class == GrayScale) ||
                                         ((attr.visual)->class == StaticGray))
    mono_flag = TRUE;

/* get the default colormap for the display */
  colormap = DefaultColormap(XtDisplay(sky_widget),
			             DefaultScreen(XtDisplay(sky_widget)));

/* for a color display, get the pixel values for each desired color name */
  if (! mono_flag)
    for (i = 0; i < NUMSTARCOLORS; i++) {
      if (! XParseColor(XtDisplay(sky_widget),colormap,
				                    starcolors[i],&rgbdef)) {
	printf("Cannot find color %s in color database\n",starcolors[i]);
	exit(0);
      }

      /* allocate a color cell in the color map */
      if (! XAllocColor(XtDisplay(sky_widget),colormap,&rgbdef)) {
	printf("Cannot allocate color cell\n");
	exit(0);
      }

      /* save the pixel value for this color */
      colorpixelval[i] = (int)rgbdef.pixel;
    }

/* create the graphics context for each star color */
  gcmask = GCForeground | GCBackground | GCArcMode;
  gcvalues.background = blackpixelval;
  gcvalues.arc_mode = ArcPieSlice;
  if (mono_flag) {
    /* monochrome monitor */
    gcvalues.foreground = whitepixelval;
    mono_star_gc = XCreateGC(XtDisplay(sky_widget),XtWindow(sky_widget),
			                                   gcmask,&gcvalues);
  }
  else
    for (i = 0; i < NUMSTARCOLORS; i++) {
      gcvalues.foreground = colorpixelval[i];
      star_gcs[i] = XCreateGC(XtDisplay(sky_widget),XtWindow(sky_widget),
			                                   gcmask,&gcvalues);
    }

/* create the graphics context for the object box */
  gcmask = GCForeground | GCBackground;
  gcvalues.background = blackpixelval;
  gcvalues.foreground = whitepixelval;
  obj_gc = XCreateGC(XtDisplay(sky_widget),XtWindow(sky_widget),
			                                   gcmask,&gcvalues);

/* for the moment, the object box GC will do fine for the grid line GC */
  grid_gc = obj_gc;

/* create the graphics context for the text box */
  gcmask = GCForeground | GCBackground | GCFunction;
  gcvalues.background = blackpixelval;
  gcvalues.foreground = whitepixelval ^ blackpixelval;
  gcvalues.function = GXxor;
  text_gc = XCreateGC(XtDisplay(sky_widget),XtWindow(sky_widget),
			                                   gcmask,&gcvalues);

/* if a font was given on the command line, use it; otherwise, grab the one
 * from the X resources */
  if (latin_font == (char *)NULL)
    latin_font = (char *)iv.latinfont;
  if (greek_font == (char *)NULL)
    greek_font = (char *)iv.greekfont;

/* load the fonts we'll use for the ID text */
  if ((latinidfontstr = XLoadQueryFont(XtDisplay(sky_widget),latin_font)) ==
                                                        (XFontStruct *)NULL) {
    printf("cannot load font %s\n",latin_font);
    exit(0);
  }

  if ((greekidfontstr = XLoadQueryFont(XtDisplay(sky_widget),greek_font)) ==
                                                        (XFontStruct *)NULL) {
    printf("cannot load font %s\n",greek_font);
    exit(0);
  }

  return;
}



/* this is the sky expose event action code */

/* ARGSUSED */

static XtActionProc sky_expose(sky_widget,event,params,num_params)

Widget sky_widget;
XExposeEvent *event;
String *params;
Cardinal *num_params;

{
  Dimension width, height;
  Display *sky_display;
  Window sky_window;

#ifdef DEBUG
  printf("\nexpose structure x, y = %d, %d\n",event->x,event->y);
  printf("expose structure width, height = %d, %d\n",event->width,
	                                                       event->height);
  printf("expose event count = %d\n",event->count);
#endif

/* save the initial position and size of the toplevel widget */
  if (! initial_values_captured) {
    XtVaGetValues(toplevel,XtNx,&old_x,XtNy,&old_y,(String *)NULL);
    XtVaGetValues(toplevel,XtNwidth,&old_width,XtNheight,&old_height,
		                                              (String *)NULL);
    initial_values_captured = TRUE;
  }

/* get the width and height of the sky area */
  XtVaGetValues(sky_widget,XtNwidth,&width,XtNheight,&height,(String *)NULL);
#ifdef DEBUG
  printf("expose event seen; width = %d, height = %d\n",width,height);
  printf("display width = %d, display height = %d\n",display.width,
	                                                      display.height);
#endif

/* if size is different, clear the screen and recalculate the object list */
  if ((height != display.height) || (width != display.width)) {
    /* clear the old stars from the window */
    sky_display = XtDisplay(sky_widget);
    sky_window = XtWindow(sky_widget);
    XClearArea(sky_display,sky_window,0,0,display.width,display.height,True);

    /* plug in new width and height */
    display.width = width;
    display.height = height;

    /* compute new display width and height in radians */
    display.width_rad = (display.width * display.scale) / DEGREES_PER_RAD;
    display.height_rad = (display.height * display.scale) / DEGREES_PER_RAD;

    /* set the "waiting" cursor */
    set_wait_cursor();

    /* generate a new object display list */
    calculate_field(&display);

    /* update the scrollbar thumbs */
    update_thumbs();
  }

  /* set the "waiting" cursor, in case it is not already set */
  set_wait_cursor();

/* now redraw the sky */
  draw_sky(event->x,event->y,event->width,event->height);

/* draw the grid lines only on the last expose event */
  if (draw_grid)
    if (event->count == 0) {
      draw_grid_lines(0,0,(int)width,(int)height);

      /* also draw the ID texts last */
      if (draw_ids)
	draw_id_texts(0,0,(int)width,(int)height);
    }
    else
      /* wait for the last expose */
      ;
  else
    /* redraw the ID texts as they are exposed */
    if (draw_ids)
      draw_id_texts(event->x,event->y,event->width,event->height);

  /* restore the normal cursor */
  set_normal_cursor();

  return;
}



/* redraw the objects in the sky widget in response to an expose event */

static void draw_sky(ul_x,ul_y,width,height)

int ul_x, ul_y;
int width, height;

{
  struct cat_header *catalog;
  struct obj_node *obj;

#ifdef DEBUG
  printf("drawing objects in area %d,%d %d,%d\n",ul_x,ul_y,width,height);
#endif

/* flag that we are not drawing in PostScript */
  ps_draw_flag = FALSE;

/* draw objects from each catalog */
  catalog = cat_list_head;
  while (catalog != (struct cat_header *)NULL) {

/* if the catalog is active, draw all objects in its display list */
    if (catalog->show_flag) {
      obj = catalog->obj_list_head;
      while (obj != (struct obj_node *)NULL) {

/* handle display of the object */
	(*catalog->draw_object)(sky_widget,obj->array_pos,obj->x,obj->y,
			                              ul_x,ul_y,width,height);

/* step to the next object */
	obj = obj->next;
      }
    }

/* step to the next catalog */
    catalog = catalog->next_cat;
  }

  return;
}



/* put out the PostScript code to draw the displayed sky */

/* ARGSUSED */

static XtCallbackProc ps_draw_sky(w,client_data,call_data)

Widget w;
XtPointer client_data;
XtPointer call_data;

{
#ifdef DEBUG
  printf("drawing objects in area %d,%d\n",display.width,display.height);
#endif

/* flag that we are drawing in PostScript */
  ps_draw_flag = TRUE;

/* open the file and write out the preamble stuff */
  if ((psfile = fopen(PS_OUTPUT_FILE,"w")) == (FILE *)NULL)
    return;

/* write the PostScript commands into the file */
  if (! ps_write_file())
    XBell(XtDisplay(sky_widget),0);

/* close the PostScript file */
  (void)fclose(psfile);

  return;
}



/* write the PostScript drawing commands */

static boolean ps_write_file()

{
  struct cat_header *catalog;
  char ps_buffer[200];
  char local_pos_buffer[50];
  struct obj_node *obj;
  struct id_node *id;

/* put out the PostScript function definitions */
  if (! ps_setup())
    return(FALSE);

/* start out the PostScript chart header */
  if (! ps_begin_header())
    return(FALSE);

/* put out the header lines */
  catalog = cat_list_head;
  while (catalog != (struct cat_header *)NULL) {
    if (catalog->show_flag) {
      sprintf(ps_buffer,"Catalog:  %s",catalog->catalog_name);
      if (! ps_put_header(ps_buffer))
	return(FALSE);
    }

    /* step to the next catalog */
    catalog = catalog->next_cat;
  }

/* output other relevant chart values */
  if (! ps_put_header("Equinox:  2000.0"))
    return(FALSE);

  sprintf(ps_buffer,"Limiting magnitude:  %.2f",mag_limit);
  if (! ps_put_header(ps_buffer))
    return(FALSE);

/* build the right ascension position in the buffer */
  sprintf(local_pos_buffer,"%2dh %02dm %02ds",display.ra_pos.hours,
	                                            display.ra_pos.minutes,
	                                            display.ra_pos.seconds);

/* add it to the PostScript buffer */
  sprintf(ps_buffer,"Chart center:  RA = %s, ",local_pos_buffer);

/* build the declination position in the buffer */
  if (display.dec_pos.sign == (short)-1)
    strcpy(local_pos_buffer,"-");
  else
    local_pos_buffer[0] = '\0';
  sprintf(&local_pos_buffer[strlen(local_pos_buffer)],"%2d~ %02d' %02d\"",
	                                            display.dec_pos.degrees,
	                                            display.dec_pos.minutes,
	                                            display.dec_pos.seconds);

/* and add it to the PostScript buffer */
  sprintf(&ps_buffer[strlen(ps_buffer)]," dec = %s",local_pos_buffer);

  /* output the chart position line */
  if (! ps_put_header(ps_buffer))
    return(FALSE);

/* if grid lines are being drawn, give the grid line separations */
  if (draw_grid) {
    if (ra_line_interval < 1000)
      sprintf(ps_buffer,"Right ascension lines are 0.%.03ds apart, ",
	                                                    ra_line_interval);
    else if (ra_line_interval < (60 * 1000))
      sprintf(ps_buffer,"Right ascension lines are %.3gs apart, ",
	                                      (float)ra_line_interval / 1000);
    else if (ra_line_interval < (60 * 60 * 1000))
      sprintf(ps_buffer,"Right ascension lines are %.3gm apart, ",
	                               (float)ra_line_interval / (60 * 1000));
    else
      sprintf(ps_buffer,"Right ascension lines are %.3gh apart, ",
	                          (float)ra_line_interval / (60 * 60 * 1000));

    if (dec_line_interval < 100)
      sprintf(&ps_buffer[strlen(ps_buffer)],
	                             "declination lines are 0.%.03d\" apart\n",
	                                                   dec_line_interval);
    else if (dec_line_interval < (60 * 100))
      sprintf(&ps_buffer[strlen(ps_buffer)],
	                               "declination lines are %.3g\" apart\n",
	                                      (float)dec_line_interval / 100);
    else if (dec_line_interval < (60 * 60 * 100))
      sprintf(&ps_buffer[strlen(ps_buffer)],
	                               "declination lines are %.3g' apart\n",
	                               (float)dec_line_interval / (60 * 100));
    else
      sprintf(&ps_buffer[strlen(ps_buffer)],
	                          "declination lines are %.3g~ apart\n",
	                          (float)dec_line_interval / (60 * 60 * 100));

    if (! ps_put_header(ps_buffer))
      return(FALSE);
  }

/* signal the end of the header information */
  if (! ps_end_header())
    return(FALSE);

/* set up to begin drawing the chart */
  if (! ps_begin_drawing((Dimension)display.width,(Dimension)display.height))
    return(FALSE);

/* draw objects from all catalogs */
  catalog = cat_list_head;
  while (catalog != (struct cat_header *)NULL) {

/* if the catalog is active, draw all the objects in its display list */
    if (catalog->show_flag) {
      obj = catalog->obj_list_head;
      while (obj != (struct obj_node *)NULL) {

/* construct and emit the PostScript commands to draw the object */
	if (! (*catalog->ps_draw_object)(obj,(Dimension)display.height))
	  return(FALSE);

/* step to the next object */
	obj = obj->next;
      }
    }

/* step to the next catalog */
    catalog = catalog->next_cat;
  }

/* output the identification strings */
  if (draw_ids) {
    id = id_list;
    while (id != (struct id_node *)NULL) {
      if (! ps_draw_id(id,(Dimension)display.height))
	return(FALSE);
      id = id->next;
    }
  }

/* draw the grid lines over the top of it all */
  if (draw_grid)
    draw_grid_lines(0,0,(int)display.width,(int)display.height);

/* end the drawing and return the final success or failure */
  return(ps_end_drawing());
}



/* draw right ascension and declination lines on the display */

/* ARGSUSED */

static XtCallbackProc handle_grid_button(menu_widget,client_data,call_data)

Widget menu_widget;
XtPointer client_data;
XtPointer call_data;

{
  int n;
  Arg buttonargs[5];

/* get the new state of the toggle button */
  n = 0;
  XtSetArg(buttonargs[n],XtNstate,&draw_grid);                            n++;
  XtGetValues(menu_widget,buttonargs,n);

#ifdef GRID_DEBUG
  printf("grid menu option called to turn grid flag ");
  if (draw_grid)
    printf("on\n");
  else
    printf("off\n");
#endif

/* draw or erase grid lines, as necessary */
  if (draw_grid) {
    /* set the "waiting" cursor */
    set_wait_cursor();

    /* set up a work procedure to do the calculations and draw the grid */
    (void)XtAddWorkProc(draw_grid_lines_work,(XtPointer)&display);
  }
  else
    /* grid lines are not currently eraseable - clear the display and redraw */
    redraw_sky();

  return;
}



/* hide the object ID texts */

/* ARGSUSED */

static XtCallbackProc toggle_ids(menu_widget,client_data,call_data)

Widget menu_widget;
XtPointer client_data;
XtPointer call_data;

{
  int n;
  Arg buttonargs[5];

/* get the new state of the toggle button */
  n = 0;
  XtSetArg(buttonargs[n],XtNstate,&draw_ids);                             n++;
  XtGetValues(menu_widget,buttonargs,n);

/* reverse the sense of the flag, which is opposite that of the toggle */
  draw_ids = ! draw_ids;

/* redraw the sky */
  redraw_sky();

  return;
}



/* erase all object ID texts */

/* ARGSUSED */

static XtCallbackProc erase_ids(menu_widget,client_data,call_data)

Widget menu_widget;
XtPointer client_data;
XtPointer call_data;

{
  struct id_node *id, *next_id;

/* free all IDs in the id list */
  id = id_list;
  while (id != (struct id_node *)NULL) {
    /* save a pointer to the next ID */
    next_id = id->next;

    /* free the ID node */
    free(id);

    /* step to the next ID */
    id = next_id;
  }

/* zap the ID list head */
  id_list = (struct id_node *)NULL;

/* redraw the sky */
  redraw_sky();

  return;
}



/* recalculate the display lists and redraw the sky display */

static void update_sky(display)

struct display *display;

{
/* set the "waiting" cursor */
  set_wait_cursor();

/* set up a work procedure to do the calculations and redraw the display */
  (void)XtAddWorkProc(update_sky_work,(XtPointer)display);

  return;
}



/* work procedure for updating sky display */

static Boolean update_sky_work(client_data)

XtPointer client_data;

{
  struct display *display = (struct display *)client_data;

/* build the new object list */
  calculate_field(display);

/* update the scrollbar thumbs */
  update_thumbs();

/* clear away the old objects and draw the new ones */
  redraw_sky();

  return(True);
}



/* unconditionally redraw the sky display */

static void redraw_sky()

{
/* clear the old objects from the window and draw the new ones via expose */
  XClearArea(XtDisplay(sky_widget),XtWindow(sky_widget),0,0,
	                                   display.width,display.height,True);

  return;
}



/* quit button callback function */

/* ARGSUSED */

static XtCallbackProc Quit(w,client_data,call_data)

Widget w;
XtPointer client_data;
XtPointer call_data;

{
  struct cat_header *catalog;

/* close all open catalog files */
  catalog = cat_list_head;
  while (catalog != (struct cat_header *)NULL) {
    if (catalog->file_fd != (FILE *)NULL)
      (void)fclose(catalog->file_fd);

    catalog = catalog->next_cat;
  }

/* just exit when the Quit function is invoked */
  exit(0);
}



/* display information window popdown (implied "cancel" button) */

/* ARGSUSED */

static XtEventHandler cancel_dpyinfo(w,client_data,event,cont_flag)

Widget w;
XtPointer client_data;
XEvent *event;
Boolean *cont_flag;

{
/* simply call the cancel button's callback */
  DpyInfoAcked(w,(XtPointer)-1,(XtPointer)NULL);

  return;
}



/* info acknowlege button callback function */

/* ARGSUSED */

static XtCallbackProc DpyInfoAcked(w,client_data,call_data)

Widget w;
XtPointer client_data;
XtPointer call_data;

{
  boolean mag_change_flag;
  boolean scale_change_flag;
  float pixels;

/* pop down the shell */
  XtPopdown(pshell);

/* resensitize the menu info button */
  XtSetSensitive(menuinfo,True);

/* if the client data is -1, ignore the edited strings and take no action */
  if (client_data == (XtPointer)-1)
    return;

/* see if the magnitude should change */
  mag_change_flag = strcmp(mag_buffer,old_mag_buffer) != EQUAL;

/* see if the scale should change */
  scale_change_flag = strcmp(scale_buffer,old_scale_buffer) != EQUAL;

/* get the new magnitude limit of the display */
  if (mag_change_flag)
    mag_limit = atof(mag_buffer);

/* calculate the new scale of the display */
  if (scale_change_flag) {
    /* get the pixels */
    pixels = atof(scale_buffer);
    display.scale = 1.0 / pixels;

    /* compute the new display width and height in radians */
    display.width_rad = (display.width * display.scale) / DEGREES_PER_RAD;
    display.height_rad = (display.height * display.scale) / DEGREES_PER_RAD;
#ifdef DEBUG
    printf("display dimensions = %lf, %lf radians\n",display.width_rad,
	                                                 display.height_rad);
#endif
  }

/* calculate the new object display list if the scale changed */
  if (scale_change_flag)
    /* update the sky display */
    update_sky(&display);
  else if (mag_change_flag)
/* clear away the old objects and draw the new ones if magnitude changed */
    redraw_sky();
  else
    ;

  return;
}



/* object info acknowlege button callback function */

/* ARGSUSED */

static XtCallbackProc DataAcked(w,client_data,call_data)

Widget w;
XtPointer client_data;
XtPointer call_data;

{
/* pop down the shell */
  XtPopdown((Widget)client_data);

  return;
}



/* object info fake ack event handler */

/* ARGSUSED */

static XtEventHandler DataAckEvent(w,client_data,event,cont_flag)

Widget w;
XtPointer client_data;
XVisibilityEvent *event;
Boolean *cont_flag;

{
/* pop down the shell only if this is a visibility obscuration event */
  if (event->state != VisibilityUnobscured)
    XtPopdown(w);

  return;
}



/* handle editing of the display center position text */

/* ARGSUSED */

static XtActionProc display_center_edited(w,event,params,num_params)

Widget w;
XEvent *event;
String *params;
Cardinal *num_params;

{
/* calculate the new position of the display */
  if (strcmp(pos_buffer,old_pos_buffer) != EQUAL) {
#ifdef DEBUG
    printf("recalculating position from buffer %s\n",pos_buffer);
#endif
    if (calculate_position(&display,pos_buffer)) {
      /* position change successful; update the sky display */
      update_center_position();
      update_sky(&display);
    }
    else {
      /* ring a bell to indicate we were unhappy about the supplied position */
      XBell(XtDisplay(sky_widget),0);

      /* and restore the old position */
      strcpy(pos_buffer,old_pos_buffer);
      XtVaSetValues(centertext,XtNstring,(String)pos_buffer,(String *)NULL);
    }
  }

  return;
}



/* handle editing of the Info display texts */

/* ARGSUSED */

static XtActionProc DpyInfoEdited(w,event,params,num_params)

Widget w;
XEvent *event;
String *params;
Cardinal *num_params;

{
/* simply convert this into an acknowledge button activate callback */
  DpyInfoAcked((Widget)NULL,(XtPointer)NULL,(XtPointer)NULL);

  return;
}



/* work procedure to draw grid lines on the sky display */

static Boolean draw_grid_lines_work(client_data)

XtPointer client_data;

{
  struct display *display = (struct display *)client_data;

/* draw the lines on the entire display */
  draw_grid_lines(0,0,(int)display->width,(int)display->height);

/* restore the normal cursor */
  set_normal_cursor();

  return(True);
}



/* draw RA and declination lines in the exposed area of the display */

static void draw_grid_lines(x,y,width,height)

int x, y;
int width, height;

{
  int lr_x, lr_y;
#if 0     /* xsky 1.0 grid code */
  int pole_x, pole_y;
  boolean north_pole_flag;
  boolean south_pole_flag;
#endif     /* xsky 1.0 grid code */
  float scale;
  int i;
  float display_width, display_height;
  int dot_intervals[300];
  int num_dot_intervals;
  int ra_dot_interval;
  int dec_dot_interval;
  int line_start_ra;
  int line_end_ra;
  int line_start_dec;
  int line_end_dec;
  int start_ra;
  int end_ra;
  int start_dec;
  int end_dec;
  boolean stop_on_greater_eq;

#ifdef GRID_DEBUG
  printf("drawing grid lines from %d,%d for width,height = %d,%d\n",
	                                                    x,y,width,height);
#endif

/* compute the lower-right coordinates of the exposed box */
  lr_x = x + width - 1;
  lr_y = y + height - 1;

/* given the upper-left X coordinates and the X width and length of the
 * exposed box, we need to draw lines of right ascension and declination.
 * the first step is to establish the east and west limits of right ascen-
 * sion and the north and south limits of declination of the box.
 *
 * for right ascension, this will occur at the corners; for declination,
 * this will occur either at the corners or in the middle.  a special
 * case occurs if either pole is in the display, in which case the RA
 * must go full-circle and the minimum or maximum declination will be
 * that pole.
 *
 * to draw the lines, we proceed eastward from the right-hand RA limit
 * to the left-hand RA limit, and northward from the southern declination
 * limit to the northern declination limit.  we place dots on the screen
 * along each line at an interval determined by the display scale; the
 * intervals between lines are also determined by display scale.
 *
 * some special cases apply here:  we want to be able to decrease the
 * number of RA lines as we approach either pole, and we draw a small
 * cross instead of a dot at the pole itself. */

#if 0    /* xsky 1.0 grid code */
/* first, set flags if either pole is in the display */
  pos_rad_to_X(&display,0.0,dec_to_rad(1,90,0,0,0),&pole_x,&pole_y);
  if ((pole_x >= 0) && (pole_x <= display.width) &&
                                  (pole_y >= 0) && (pole_y <= display.height))
    north_pole_flag = TRUE;
  else
    north_pole_flag = FALSE;

  pos_rad_to_X(&display,0.0,dec_to_rad(-1,90,0,0,0),&pole_x,&pole_y);
  if ((pole_x >= 0) && (pole_x <= display.width) &&
                                  (pole_y >= 0) && (pole_y <= display.height))
    south_pole_flag = TRUE;
  else
    south_pole_flag = FALSE;

#ifdef GRID_DEBUG
  if (north_pole_flag)
    printf("north celestial pole is in the display\n");
  if (south_pole_flag)
    printf("south celestial pole is in the display\n");
#endif

/* if both poles are in the display, the limits are easy */
  if (north_pole_flag && south_pole_flag) {
    start_ra = 1;
    end_ra = 0;
    start_dec = -90 * 60 * 60 * 100;
    end_dec = 90 * 60 * 60 * 100;
  }
  else if (north_pole_flag) {
    start_ra = 1;
    end_ra = 0;
    start_dec = find_south_dec(x,y,lr_x,lr_y);
    end_dec = 90 * 60 * 60 * 100;
  }
  else if (south_pole_flag) {
    start_ra = 1;
    end_ra = 0;
    start_dec = -90 * 60 * 60 * 100;
    end_dec = find_north_dec(x,y,lr_x,lr_y);
  }
  else {
    start_ra = find_west_ra(x,y,lr_x,lr_y);
    end_ra = find_east_ra(x,y,lr_x,lr_y);
    start_dec = find_south_dec(x,y,lr_x,lr_y);
    end_dec = find_north_dec(x,y,lr_x,lr_y);
  }
#else      /* xsky 1.0 grid code */
/* convert the display limits to integral units */
  start_ra = ra_rad_to_thousandths(display.right_ra_rad);
  end_ra = ra_rad_to_thousandths(display.left_ra_rad);
  start_dec = dec_rad_to_hundredths(display.bottom_dec_rad);
  end_dec = dec_rad_to_hundredths(display.top_dec_rad);
#endif     /* xsky 1.0 grid code */

#ifdef GRID_DEBUG
  printf("start ra  = %d, end ra  = %d\n",start_ra,end_ra);
  printf("start dec = %d, end dec = %d\n",start_dec,end_dec);
#endif

/* convert the display scale to pixels per degree */
  scale = 1 / display.scale;

/* determine the line intervals and dot intervals based on the display scale */
/* if the grid interval table is not yet built, build it now */
  if (! interval_table_built) {
    num_grid_intervals = build_grid_intervals(grid_intervals,60 * 60 * 1000);
    interval_table_built = TRUE;
  }

/* choose the largest grid line interval that divides the display into more
 * than three parts horizontally and vertically */
  display_width = ((display.width / scale) / DEGREES_PER_HOUR) *
                                                        60 * 60 * 1000;
  i = 0;
  while (i < num_grid_intervals) {
    if ((display_width / grid_intervals[i]) / cos(display.center_dec_rad)
	                                                               <= 3.0)
      break;
    else
      i++;
#ifdef DEBUG
    printf("grid interval %d divides the width %f times\n",grid_intervals[i],
	                                   display_width / grid_intervals[i]);
#endif
  }
  if (i > 0)
    i--;
  ra_line_interval = grid_intervals[i];

  /* do the same thing in the vertical direction */
  display_height = (display.height / scale) * 60 * 60 * 100;
  i = 0;
  while (i < num_grid_intervals) {
    if ((display_height / grid_intervals[i]) <= 3.0)
      break;
    else
      i++;
#ifdef DEBUG
    printf("grid interval %d divides the height %f times\n",grid_intervals[i],
	                                  display_height / grid_intervals[i]);
#endif
  }
  if (i > 0)
    i--;
  dec_line_interval = grid_intervals[i];

/* choose the largest dot interval that provides at least 40 dots per grid
 * interval */
  num_dot_intervals = build_grid_intervals(dot_intervals,ra_line_interval);
  i = 0;
  while (i < num_dot_intervals)
    if ((ra_line_interval / dot_intervals[i]) < 40)
      break;
    else
      i++;
  if (i > 0)
    i--;
  ra_dot_interval = dot_intervals[i];

  /* do the same for the declination dot interval */
  num_dot_intervals = build_grid_intervals(dot_intervals,dec_line_interval);
  i = 0;
  while (i < num_dot_intervals)
    if ((dec_line_interval / dot_intervals[i]) < 40)
      break;
    else
      i++;
  if (i > 0)
    i--;
  dec_dot_interval = dot_intervals[i];

#ifdef DEBUG
  printf("scale = %f\n",scale);
  printf("display width = %f; display height = %f\n",display_width,
                                                              display_height);
  printf("RA grid interval = %d; dec. grid interval = %d\n",ra_line_interval,
                                                           dec_line_interval);
  printf("RA dot interval = %d; dec. dot interval = %d\n",ra_dot_interval,
                                                           dec_dot_interval);
#endif

/* round the starting right ascension to the next easterly line */
  line_start_ra = (start_ra + ra_line_interval) / ra_line_interval *
                                                             ra_line_interval;

/* round the ending right ascension to the next westerly line */
  line_end_ra = end_ra / ra_line_interval * ra_line_interval;

/* round the starting right ascension to the next dot interval */
  start_ra = (start_ra + ra_dot_interval) / ra_dot_interval * ra_dot_interval;

/* round the starting declination to the next northerly line */
  if (start_dec < 0)
    line_start_dec = start_dec / dec_line_interval * dec_line_interval;
  else
    line_start_dec = (start_dec + dec_line_interval) / dec_line_interval *
                                                            dec_line_interval;

/* round the ending declination to the next southerly line */
  if (end_dec < 0)
    line_end_dec = (end_dec - dec_line_interval) / dec_line_interval *
                                                            dec_line_interval;
  else
    line_end_dec = end_dec / dec_line_interval * dec_line_interval;

/* round the starting declination to the next northerly dot interval */
  if (start_dec < 0)
    start_dec = start_dec / dec_dot_interval * dec_dot_interval;
  else
    start_dec = (start_dec + dec_dot_interval) / dec_dot_interval *
                                                             dec_dot_interval;

/* set a flag if we're not going through the 24-hour line */
  stop_on_greater_eq = (boolean)(line_start_ra <= line_end_ra);

/* draw lines of right ascension */
  while (TRUE) {
    /* draw this line of right ascension */
    draw_ra_line(line_start_ra,start_dec,end_dec,ra_dot_interval,
		                                               x,y,lr_x,lr_y);

    /* check to see if we're done */
    if ((line_start_ra >= line_end_ra) && stop_on_greater_eq)
      break;

    /* step to the next RA line, being careful when we pass 24 hours */
    line_start_ra += ra_line_interval;
    if (line_start_ra > (24 * 60 * 60 * 1000)) {
      line_start_ra -= 24 * 60 * 60 * 1000;
      stop_on_greater_eq = TRUE;
    }
  };

/* draw lines of declination */
  while (line_start_dec <= line_end_dec) {
    draw_dec_line(start_ra,end_ra,line_start_dec,dec_dot_interval,
		                                               x,y,lr_x,lr_y);
    /* step to the next declination line */
    line_start_dec += dec_line_interval;
  }

  return;
}



#if 0     /* xsky 1.0 grid code */

/* return the easternmost right ascension of the exposed area in radians */

/* ARGSUSED */

static int find_east_ra(ul_x,ul_y,lr_x,lr_y)

int ul_x, ul_y;
int lr_x, lr_y;

{
  double ul_corner_rad, ll_corner_rad;
  double dummy_dec_rad;

/* get the right ascension of the upper-left corner */
  X_to_pos_rad(&display,ul_x,ul_y,&ul_corner_rad,&dummy_dec_rad);

/* get the right ascension of the lower-left corner */
  X_to_pos_rad(&display,ul_x,lr_y,&ll_corner_rad,&dummy_dec_rad);

/* if the center of the display is in the northern hemisphere, then the upper-
 * left corner is the ending RA */
  if (display.dec_pos.sign == 1)
    return(ra_rad_to_thousandths(ul_corner_rad));
  else
    return(ra_rad_to_thousandths(ll_corner_rad));
}



/* return the westernmost right ascension of the exposed area radians */

/* ARGSUSED */

static int find_west_ra(ul_x,ul_y,lr_x,lr_y)

int ul_x, ul_y;
int lr_x, lr_y;

{
  double ur_corner_rad, lr_corner_rad;
  double dummy_dec_rad;

/* get the right ascension of the upper-right corner */
  X_to_pos_rad(&display,lr_x,ul_y,&ur_corner_rad,&dummy_dec_rad);

/* get the right ascension of the lower-right corner */
  X_to_pos_rad(&display,lr_x,lr_y,&lr_corner_rad,&dummy_dec_rad);

#ifdef GRID_DEBUG
  printf("finding west RA with ul_x,ul_y = %d,%d\n",ul_x,ul_y);
  printf("and lr_x,lr_y = %d,%d\n",lr_x,lr_y);
  printf("ur_corner_rad = %lf, lr_corner_rad = %lf\n",ur_corner_rad,
	                                                      lr_corner_rad);
#endif

/* if the center of the display is in the northern hemisphere, then the upper-
 * right corner is tne starting RA */
  if (display.dec_pos.sign == 1)
    return(ra_rad_to_thousandths(ur_corner_rad));
  else
    return(ra_rad_to_thousandths(lr_corner_rad));
}



/* return the northernmost declination of the exposed area in hundredths */

/* ARGSUSED */

static int find_north_dec(ul_x,ul_y,lr_x,lr_y)

int ul_x, ul_y;
int lr_x, lr_y;

{
  double corner_dec_rad, center_dec_rad;
  double dummy_ra_rad;

/* get the declination of the upper-left corner */
  X_to_pos_rad(&display,ul_x,ul_y,&dummy_ra_rad,&corner_dec_rad);

/* get the declination of the top center */
  X_to_pos_rad(&display,(lr_x - ul_x) / 2,ul_y,&dummy_ra_rad,&center_dec_rad);

/* return the value of the greater declination */
  return(dec_rad_to_hundredths(max(corner_dec_rad,center_dec_rad)));
}



/* return the southernmost declination of the exposed area in hundredths */

/* ARGSUSED */

static int find_south_dec(ul_x,ul_y,lr_x,lr_y)

int ul_x, ul_y;
int lr_x, lr_y;

{
  double corner_dec_rad, center_dec_rad;
  double dummy_ra_rad;

/* get the declination of the lower-right corner */
  X_to_pos_rad(&display,lr_x,lr_y,&dummy_ra_rad,&corner_dec_rad);

/* get the declination of the bottom center */
  X_to_pos_rad(&display,(lr_x - ul_x) / 2,lr_y,&dummy_ra_rad,&center_dec_rad);

/* return the value of the lesser declination */
  return(dec_rad_to_hundredths(min(corner_dec_rad,center_dec_rad)));
}

#endif     /* xsky 1.0 grid code */



/* draw one line of right ascension for the RA/declination grid */

static void draw_ra_line(ra,start_dec,end_dec,dot_interval,ul_x,ul_y,lr_x,lr_y)

int ra;
int start_dec;
int end_dec;
int dot_interval;
int ul_x, ul_y;
int lr_x, lr_y;

{
/* if this is PostScript, begin a new path */
  if (ps_draw_flag)
    ps_grid_newpath();

/* draw dots until we pass the ending declination */
  while (start_dec <= end_dec) {
    draw_grid_dot(ra,start_dec,ul_x,ul_y,lr_x,lr_y);

    /* step to the next dot */
    start_dec += dot_interval;
  }

/* if this is PostScript, end the path */
  if (ps_draw_flag)
    ps_grid_endpath();

  return;
}



/* draw one line of declination for the RA/declination grid */

static void draw_dec_line(start_ra,end_ra,dec,dot_interval,ul_x,ul_y,lr_x,lr_y)

int start_ra;
int end_ra;
int dec;
int dot_interval;
int ul_x, ul_y;
int lr_x, lr_y;

{
  boolean stop_on_greater_eq;

/* don't draw anything at either pole */
  if (abs(dec) == (90 * 60 * 60 * 100))
    return;

/* if this is PostScript, begin a new path */
  if (ps_draw_flag)
    ps_grid_newpath();

/* set a flag if we're not going through the 24-hour line */
  stop_on_greater_eq = (boolean)(start_ra <= end_ra);

/* draw grid dots for this declination line */
  while (TRUE) {
    /* draw one dot along the declination line */
    draw_grid_dot(start_ra,dec,ul_x,ul_y,lr_x,lr_y);

    /* check to see if we're done */
    if ((start_ra >= end_ra) && stop_on_greater_eq)
      break;

    /* step to the next dot, being careful when we pass 24 hours */
    start_ra += dot_interval;
    if (start_ra > (24 * 60 * 60 * 1000)) {
      start_ra -= 24 * 60 * 60 * 1000;
      stop_on_greater_eq = TRUE;
    }
  };

/* if this is PostScript, end the path */
  if (ps_draw_flag)
    ps_grid_endpath();

  return;
}



/* draw one dot for the right ascension/declination grid */

static void draw_grid_dot(ra_thousandths,dec_hundredths,ul_x,ul_y,lr_x,lr_y)

int ra_thousandths;
int dec_hundredths;
int ul_x, ul_y;
int lr_x, lr_y;

{
  int x, y;
  float ps_x, ps_y;

/* convert the position to floating X coordinates */
  pos_rad_to_X_float(&display,ra_thousandths_to_rad(ra_thousandths),
	                        dec_hundredths_to_rad(dec_hundredths),
		                                                 &ps_x,&ps_y);

/* convert floating X coordinates into integer pixel positions */
  x = ps_x + 0.5;
  y = ps_y + 0.5;

/* if the dot is within the area, draw it */
  if ((x >= ul_x) && (x <= lr_x) && (y >= ul_y) && (y <= lr_y))
    if (ps_draw_flag)
      ps_draw_point((double)ps_x,(double)ps_y,(Dimension)display.height);
    else
      draw_point(sky_widget,(Position)x,(Position)y);

  return;
}



/* object selection window popdown (implied "cancel" button) */

/* ARGSUSED */

static XtEventHandler cancel_find(w,client_data,event,cont_flag)

Widget w;
XtPointer client_data;
XEvent *event;
Boolean *cont_flag;

{
/* simply call the cancel button's callback */
  FindObject(w,(XtPointer)-1,(XtPointer)NULL);

  return;
}



/* object selection acknowlege/cancel button callback function */

/* ARGSUSED */

static XtCallbackProc FindObject(w,client_data,call_data)

Widget w;
XtPointer client_data;
XtPointer call_data;

{
/* pop down the shell */
  XtPopdown(findshell);

/* resensitize the menu info button */
  XtSetSensitive(menufind,True);

/* if the client data is -1, ignore the edited string and take no action */
  if (client_data == (XtPointer)-1)
    return;

/* find the object and update the display structure */
  if (! find_object_by_name(object_buffer)) {
    XBell(XtDisplay(sky_widget),0);
    return;
  }

/* update the display center position */
  update_center_position();

/* update the sky display */
  update_sky(&display);

  return;
}



/* find the object in response to a <return> in the text field */

/* ARGSUSED */

static XtActionProc find_object_in_text(w,event,params,num_params)

Widget w;
XEvent *event;
String *params;
Cardinal *num_params;

{
  FindObject((Widget)NULL,(XtPointer)0,(XtPointer)NULL);

  return;
}



/* decode the object name and try to find the named object */

static boolean find_object_by_name(buffer)

char *buffer;

{
  char *ptr1, *ptr2;
  struct cat_header *catalog;

/* skip leading white space */
  while (isspace(*buffer))
    buffer++;

/* squeeze multiple spaces and tabs, and trailing spaces, out of the buffer */
  ptr1 = ptr2 = buffer;
  ptr2++;
  while (*ptr2 != '\0')
    if (isspace(*ptr2)) {
      if (*ptr1 == ' ')
	;
      else
	*++ptr1 = ' ';
      ptr2++;
    }
    else
      *++ptr1 = *ptr2++;

  /* if the trailing character is a space, overwrite it */
  if (*ptr1 == ' ')
    ;
  else
    ptr1++;
  *ptr1 = '\0';

/* ensure there is something in the buffer */
  if (*buffer == '\0')
    return(FALSE);

/* hand the object designation to each active catalog */
  catalog = cat_list_head;
  while (catalog != (struct cat_header *)NULL) {
    if (catalog->show_flag && catalog->find_object != NULLBOOLFUNCTION)
      if ((*catalog->find_object)(buffer))
	break;

    /* step to tne next catalog */
    catalog = catalog->next_cat;
  }

/* do the user a favor and check inactive catalogs if not found */
  if (catalog == (struct cat_header *)NULL) {
    catalog = cat_list_head;
    while (catalog != (struct cat_header *)NULL) {
      if ((! catalog->show_flag) && catalog->find_object != NULLBOOLFUNCTION)
	if ((*catalog->find_object)(buffer))
	  break;

      /* step to tne next catalog */
      catalog = catalog->next_cat;
    }
  }

/* return TRUE or FALSE depending on whether the object was found */
  return(catalog != (struct cat_header *)NULL);
}



/* info button callback function */

/* ARGSUSED */

static XtCallbackProc PrepareInfo(w,client_data,call_data)

Widget w;
XtPointer client_data;
XtPointer call_data;

{
  Position x, y;
  float display_width, display_height;

/* start out the info display with monochrome mode info */
  if (mono_flag)
    strcpy(info_display,"monochrome mode; ");
  else
    info_display[0] = '\0';

/* calculate and format the display size */
  display_width = min(display.width * display.scale,360.0);
  display_height = min(display.height * display.scale,360.0);
  sprintf(&info_display[strlen(info_display)],
	          "The display is %.3g degrees wide by %.3g degrees high\n",
	                                                      display_width,
	                                                      display_height);

/* if there is a grid showing, give the grid line intervals */
  if (draw_grid) {
    if (ra_line_interval < 1000)
      sprintf(&info_display[strlen(info_display)],
	                               "RA lines are 0.%.03ds apart, ",
	                                                    ra_line_interval);
    else if (ra_line_interval < (60 * 1000))
      sprintf(&info_display[strlen(info_display)],
	                                      "RA lines are %.3gs apart, ",
	                                      (float)ra_line_interval / 1000);
    else if (ra_line_interval < (60 * 60 * 1000))
      sprintf(&info_display[strlen(info_display)],
	                               "RA lines are %.3gm apart, ",
	                               (float)ra_line_interval / (60 * 1000));
    else
      sprintf(&info_display[strlen(info_display)],
	                          "RA lines are %.3gh apart, ",
	                          (float)ra_line_interval / (60 * 60 * 1000));
    if (dec_line_interval < 100)
      sprintf(&info_display[strlen(info_display)],
	                               "dec. lines are 0.%.03d\" apart\n",
	                                                   dec_line_interval);
    else if (dec_line_interval < (60 * 100))
      sprintf(&info_display[strlen(info_display)],
	                                      "dec. lines are %.3g\" apart\n",
	                                      (float)dec_line_interval / 100);
    else if (dec_line_interval < (60 * 60 * 100))
      sprintf(&info_display[strlen(info_display)],
	                               "dec. lines are %.3g' apart\n",
	                               (float)dec_line_interval / (60 * 100));
    else
      sprintf(&info_display[strlen(info_display)],
	                          "dec. lines are %.3gd apart\n",
	                          (float)dec_line_interval / (60 * 60 * 100));
  }
  else
    strcat(info_display,"\n");

/* initialize the editable magnitude buffer */
  sprintf(mag_buffer,"%.1f\n",mag_limit);

/* initialize the editable scale buffer */
  sprintf(scale_buffer,"%.1f",1 / display.scale);

/* save copies of the editable buffers for later comparison */
  strcpy(old_mag_buffer,mag_buffer);
  strcpy(old_scale_buffer,scale_buffer);

/* set the label of the label widget to the info string */
  XtVaSetValues(infolabel,XtNlabel,(String)info_display,(String *)NULL);

/* set the values of the magnitude and scale text widgets */
  XtVaSetValues(displaymag,XtNstring,(String)mag_buffer,(String *)NULL);
  XtVaSetValues(displayscale,XtNstring,(String)scale_buffer,(String *)NULL);

/* calculate the position of the popup shell */
  XtTranslateCoords(toplevel,100,0,&x,&y);

/* move the popup shell to this position */
  XtVaSetValues(pshell,XtNx,x,XtNy,y,(String *)NULL);

/* pop up the info display */
  XtPopup(pshell,XtGrabExclusive);

  return;
}



/* select button callback function */

/* ARGSUSED */

static XtCallbackProc PrepareFind(w,client_data,call_data)

Widget w;
XtPointer client_data;
XtPointer call_data;

{
  Position x, y;

/* set the value of the object selection text widgets */
  XtVaSetValues(findtext,XtNstring,(String)object_buffer,(String *)NULL);

/* calculate the position of the popup shell */
  XtTranslateCoords(toplevel,200,0,&x,&y);

/* move the popup shell to this position */
  XtVaSetValues(findshell,XtNx,x,XtNy,y,(String *)NULL);

/* pop up the find display */
  XtPopup(findshell,XtGrabExclusive);

  return;
}



/* handle button presses in the sky widget */

/* ARGSUSED */

static XtEventHandler HandleSkyButtonPress(w,dummy1,event,dummy2)

Widget w;
XtPointer dummy1;
XButtonPressedEvent *event;
Boolean *dummy2;

{
/* if button 1 was pressed, display star information */
  if (event->button == Button1)
    displayobjinfo(event);
  else if (event->button == Button2)
    /* if button 2 was pressed, ... */
    if ((event->state & ShiftMask) == 0)
      /* unshifted - identify the selected object */
      identify_object(event->x,event->y);
    else
      /* shifted - remove an ID string */
      remove_identification(event->x,event->y);
  else if (event->button == Button3)
/* if button 3 was pressed, recenter the display */
    recenter_display(event);
  else
    /* ignore any other button-press events */
    ;

  return;
}



/* handle button releases in the sky widget */

/* ARGSUSED */

static XtEventHandler HandleSkyButtonRelease(w,id,event,cont_flag)

Widget w;
struct id_node *id;
XButtonReleasedEvent *event;
Boolean *cont_flag;

{
/* the only one we're interested in currently is button 2 */
  if (event->button == Button2) {
/* remove event handlers */
    XtRemoveEventHandler(sky_widget,Button2MotionMask,True,
			 (XtEventHandler)handle_button2_motion,(XtPointer)id);
    XtRemoveEventHandler(sky_widget,ButtonReleaseMask,True,
			(XtEventHandler)HandleSkyButtonRelease,(XtPointer)id);

/* remove the locator box */
    draw_text_box(id->ul_x,id->ul_y,id->lr_x,id->lr_y);

/* save the final position of the box */
    id->ul_x = event->x;
    id->ul_y = event->y;

/* calculate the final lower-right position of the box */
    id->lr_x += id->ul_x;
    id->lr_y += id->ul_y;

/* calculate the location of the ID text */
    id->x += event->x;
    id->y += event->y;

/* draw the ID string and add it to the ID display list */
    draw_id_text(sky_widget,id->id_buffer,id->font,id->x,id->y);
    add_id_to_list(id);

/* and restore the normal cursor */
    set_normal_cursor();
  }
  else
    /* ignore any other button-release events */
    ;

  return;
}



/* display information about the object clicked on */

static void displayobjinfo(event)

XButtonPressedEvent *event;

{
  struct obj_node *obj;
  struct cat_header *catalog;
  Position x, y;

/* find the closest object; if none close enough, just ignore the event */
  if ((obj = find_object(event->x,event->y,&catalog)) ==
                                                    (struct obj_node *)NULL) {
    XBell(XtDisplay(sky_widget),0);
    return;
  }

#if 0
  /* calculate the position of the popup shell */
  XtTranslateCoords(sky_widget,event->x + 20,event->y,&x,&y);
#else
  /* calculate the position of the popup shell */
  XtTranslateCoords(sky_widget,0,0,&x,&y);
#endif

/* find, format, and display this object's info */
  (*catalog->display_data)(catalog->file_fd,obj->array_pos);

  /* set the label of the label widget to the info string */
  XtVaSetValues(catalog->datadialog,XtNlabel,(String)obj_info,(String *)NULL);

  /* move the popup shell into position */
  XtVaSetValues(catalog->datashell,XtNx,x,XtNy,y,(String *)NULL);

  XtPopup(catalog->datashell,XtGrabNonexclusive);

  return;
}



/* emit an identification for the selected object */

static void identify_object(x,y)

int x, y;

{
  struct obj_node *obj;
  struct cat_header *catalog;
  struct id_node *id;
  int dummy;
  XCharStruct overall_size;

/* if ID texts are not visible, don't allow the user to create any */
  if (! draw_ids) {
    XBell(XtDisplay(sky_widget),0);
    return;
  }

/* find the selected object */
  if ((obj = find_object(x,y,&catalog)) == (struct obj_node *)NULL) {
    XBell(XtDisplay(sky_widget),0);
    return;
  }

/* get an ID node for this object */
  id = (struct id_node *)malloc(sizeof(struct id_node));

/* build the identifying text (assume Latin font) */
  id->font = LATIN;
  (*catalog->format_name)(obj->array_pos,id->id_buffer,&id->font);

/* if the object has no designation, we're done */
  if (strlen(id->id_buffer) == 0) {
    free(id);
    return;
  }

/* set this ID's displayable flag */
  id->show_flag = TRUE;

/* save the object's catalog and index */
  id->obj_catalog = catalog;
  id->obj_index = obj->array_pos;

/* save the object's X position */
  id->obj_x = obj->x;
  id->obj_y = obj->y;

/* calculate the size of the identifying text */
  if (id->font == LATIN)
    XTextExtents(latinidfontstr,id->id_buffer,strlen(id->id_buffer),
		                                     &dummy,&dummy,&dummy,
	                                                       &overall_size);
  else if (id->font == GREEK)
    XTextExtents(greekidfontstr,id->id_buffer,strlen(id->id_buffer),
		                                     &dummy,&dummy,&dummy,
	                                                       &overall_size);
  else
    /* default to Latin */
    XTextExtents(latinidfontstr,id->id_buffer,strlen(id->id_buffer),
		                                     &dummy,&dummy,&dummy,
	                                                       &overall_size);

  /* save the ID text position as an offset for now */
  id->x = 0;
  id->y = overall_size.ascent;

  /* save the box upper-left position */
  id->ul_x = x;
  id->ul_y = y;

  /* remember the lower-right position as an offset for now */
  id->lr_x = overall_size.width;
  id->lr_y = overall_size.ascent + overall_size.descent;

/* register for mouse motion events under button 2 */
  XtAddEventHandler(sky_widget,Button2MotionMask,False,
		         (XtEventHandler)handle_button2_motion,(XtPointer)id);
/* register for button release events */
  XtAddEventHandler(sky_widget,ButtonReleaseMask,False,
		        (XtEventHandler)HandleSkyButtonRelease,(XtPointer)id);

/* set the text box positioning cursor */
  set_textbox_cursor();

/* draw the initial box */
  draw_text_box(x,y,id->lr_x,id->lr_y);

  return;
}



/* find the object within delta of the given position */

static struct obj_node *find_object(x,y,catalog)

int x, y;
struct cat_header **catalog;

{
  struct cat_header *cat;
  struct obj_node *obj;

/* figure out which object to display information about */
  cat = cat_list_head;
  while (cat != (struct cat_header *)NULL) {

    /* search the displayed objects list for this catalog */
    obj = cat->obj_list_head;
    while (obj != (struct obj_node *)NULL) {

      /* ignore objects which don't meet the magnitude limit */
      if (cat->mag_limit_check != NULLBOOLFUNCTION)
	if (! (*cat->mag_limit_check)(obj->array_pos,mag_limit)) {
	  /* just step to the next object */
	  obj = obj->next;
	  continue;
	}

      /* check for a click within sight of an object */
      if (sqrt((double)((x - obj->x) * (x - obj->x) +
	                       (y - obj->y) * (y - obj->y))) <= OBJ_PROXIMITY)
	/* use this object */
	break;
      else
	/* step to the next object */
	obj = obj->next;
    }

    /* if we exhausted the list, step to the next catalog */
    if (obj == (struct obj_node *)NULL)
      cat = cat->next_cat;
    else
      /* we found a close-enough object; use it */
      break;
  }

/* tell the caller which catalog we found the object in */
  *catalog = cat;

/* and return a pointer to the object found */
  return(obj);
}



/* handle motion in sky widget with button 2 pressed */

/* ARGSUSED */

static XtEventHandler handle_button2_motion(w,id,event,cont_flag)

Widget w;
struct id_node *id;
XMotionEvent *event;
Boolean *cont_flag;

{
/* redraw the current text box to erase it */
  draw_text_box(id->ul_x,id->ul_y,id->lr_x,id->lr_y);

/* update the position to the new location */
  id->ul_x = event->x;
  id->ul_y = event->y;

/* draw the new text box */
  draw_text_box(id->ul_x,id->ul_y,id->lr_x,id->lr_y);

  return;
}



/* draw the surrounding box for object ID text */

static void draw_text_box(x,y,wid,len)

int x, y;
int wid, len;

{
  Display *sky_display;
  Window sky_window;

/* get the display for the widget */
  sky_display = XtDisplay(sky_widget);

/* get the window for the widget */
  sky_window = XtWindow(sky_widget);

/* set the foreground color to be the XOR of foreground and background */
  XSetForeground(sky_display,text_gc,whitepixelval ^ blackpixelval);

/* use the XOR function to reversibly scribble over any overlaid objects */
  XSetFunction(sky_display,text_gc,GXxor);

/* draw the box surrounding the ID text */
  XDrawRectangle(sky_display,sky_window,text_gc,x,y,wid,len);

  return;
}



/* draw all identification strings in the list that fall in the given box */

static void draw_id_texts(x,y,width,height)

int x, y;
int width, height;

{
  struct id_node *id;

/* step through the ID list, redrawing all ID strings */
  id = id_list;
  while (id != (struct id_node *)NULL) {
    /* display the ID only if enabled */
    if (id->show_flag)
      /* see of the exposed area includes part of this text */
      if ((id->ul_x <= x) && (id->lr_x >= x) ||
	          (id->ul_x <= (x + width)) && (id->lr_x >= (x + width)) ||
	                         (id->ul_x >= x) && (id->lr_x <= (x + width)))
	/* the x dimension overlaps */
	if ((id->ul_y <= y) && (id->lr_y >= y) ||
	        (id->ul_y <= (y + height)) && (id->lr_y >= (y + height)) ||
	                        (id->ul_y >= y) && (id->lr_y <= (y + height)))
	  /* the y dimension overlaps, too */
	  draw_id_text(sky_widget,id->id_buffer,id->font,id->x,id->y);

    /* step to the next ID */
    id = id->next;
  }

  return;
}



/* add the ID text to the ID display list */

static void add_id_to_list(id)

struct id_node *id;

{
/* link to the front of the visible ID list */
  id->next = (struct id_node *)id_list;
  id_list = id;

  return;
}



/* remove the identification text clicked on */

static void remove_identification(x,y)

int x, y;

{
  struct id_node *id, *prev_id;

/* if the user can't see the IDs, don't let him remove them */
  if (! draw_ids) {
    XBell(XtDisplay(sky_widget),0);
    return;
  }

/* loop through all displayed IDs, looking for one enclosing (x,y) */
  prev_id = (struct id_node *)NULL;
  id = id_list;
  while (id != (struct id_node *)NULL)
    /* does this ID enclose the point? */
    if ((x > id->ul_x) && (x < id->lr_x) && (y > id->ul_y) && (y < id->lr_y)) {
      /* yes, remove it from the list and erase it from the screen */
      remove_this_id(prev_id,id);
      break;
    }
    else {
      /* step to the next ID node */
      prev_id = id;
      id = id->next;
    }

/* sound a bell if we didn't find an enclosing ID */
  if (id == (struct id_node *)NULL)
    XBell(XtDisplay(sky_widget),0);
  else
    /* redraw all the IDs we may have obscured */
    draw_id_texts(id->ul_x,id->ul_y,id->lr_x - id->ul_x,id->lr_y - id->ul_y);

  return;
}



/* remove an ID from the list and clear the text from the screen */

static void remove_this_id(prev_id,id)

struct id_node *prev_id, *id;

{
/* remove this ID from the list */
  if (prev_id == (struct id_node *)NULL)
    id_list = id->next;
  else
    prev_id->next = id->next;

/* clear the area taken up by the string and generate an expose for it */
  XClearArea(XtDisplay(sky_widget),XtWindow(sky_widget),id->ul_x,id->ul_y,
		                id->lr_x - id->ul_x,id->lr_y - id->ul_y,True);

/* free the ID node's memory */
  free(id);

  return;
}



/* recenter the display at the point of the button click */

static void recenter_display(event)

XButtonPressedEvent *event;

{
/* update the display center RA and declination */
  X_to_pos_rad(&display,event->x,event->y,&display.center_ra_rad,
	                                            &display.center_dec_rad);
  update_center_position();

/* update the sky display */
  update_sky(&display);

  return;
}



/* reformat the display position */

static void update_center_position()

{
/* convert the position in radians to sexigesimal< and round to minutes */
  display.ra_pos = rad_to_ra(display.center_ra_rad);
  display.dec_pos = rad_to_dec(display.center_dec_rad);

#ifdef ROUND_DISPLAY_POS
/* round the display position for neatness */
  round_ra(&display.ra_pos);
  round_dec(&display.dec_pos);
#endif

/* convert back to radians to handle the rounding */
  display.center_ra_rad = ra_to_rad((int)display.ra_pos.hours,
				             (int)display.ra_pos.minutes,
				             (int)display.ra_pos.seconds,
				             (int)display.ra_pos.thousandths);
  display.center_dec_rad = dec_to_rad((int)display.dec_pos.sign,
				             (int)display.dec_pos.degrees,
				             (int)display.dec_pos.minutes,
				             (int)display.dec_pos.seconds,
				             (int)display.dec_pos.hundredths);

/* update the editable display center position buffer */
  if (display.dec_pos.sign == (short)1)
    sprintf(pos_buffer,"%02dh %02dm %02ds.%.03d, %02dd %02d' %02d\".%.02d",
	                                          display.ra_pos.hours,
	                                          display.ra_pos.minutes,
	                                          display.ra_pos.seconds,
	                                          display.ra_pos.thousandths,
	                                          display.dec_pos.degrees,
	                                          display.dec_pos.minutes,
	                                          display.dec_pos.seconds,
	                                          display.dec_pos.hundredths);
  else
    sprintf(pos_buffer,"%02dh %02dm %02ds.%.03d, -%02dd %02d' %02d\".%.02d",
	                                          display.ra_pos.hours,
	                                          display.ra_pos.minutes,
	                                          display.ra_pos.seconds,
	                                          display.ra_pos.thousandths,
	                                          display.dec_pos.degrees,
	                                          display.dec_pos.minutes,
	                                          display.dec_pos.seconds,
	                                          display.dec_pos.hundredths);

/* and copy it for later comparison */
  strcpy(old_pos_buffer,pos_buffer);

/* set the display center position into the center text widget */
  XtVaSetValues(centertext,XtNstring,(String)pos_buffer,(String *)NULL);

  return;
}



/* update the position and length of the scrollbar thumbs */

static void update_thumbs()

{
  Dimension scrollsize;
  int position;
  float pos;
  XtArgVal *posptr;
  float length;
  XtArgVal *lengthptr;

/* get the length of the RA scrollbar */
  XtVaGetValues(horiz_scroll,XtNwidth,&scrollsize,(String *)NULL);

/* map the current RA onto the scrollbar */

  /*
   * this is a lot less hairy than it looks - the mapping is a linear one,
   * so we can use the usual y = mx + b.  we know that we want 2 * PI units
   * to map onto the scrollbar size, so the slope of this line (m) is just
   * (- (scrollsize / (2 * PI))) (the slope is negative because RA increases
   * to the left).  b = y - mx, and we know that when x is the initial RA.
   * we want y to be (scrollsize / 2).  substituting that x and y into the
   * formula for b yields the second term of the formula below.
   */
  position = (- (scrollsize / (2 * PI))) * display.center_ra_rad +
                       (scrollsize / 2) * (1 + (display.initial_ra_rad / PI));

/* remap to within scrollbar range */
  if (position < 0)
    position += scrollsize;
  else if (position > scrollsize)
    position -= scrollsize;

/* convert position to a percentage */
  pos = (float)position / scrollsize;
  posptr = (XtArgVal *)&pos;

/* calculate the length of the thumb */
  length = (display.width * display.scale) / 360;
  lengthptr = (XtArgVal *)&length;

/* set the new position and length */
  XtVaSetValues(horiz_scroll,XtNtopOfThumb,*posptr,XtNshown,*lengthptr,
		                                               (String *)NULL);

/* get the length of the dec scrollbar */
  XtVaGetValues(vert_scroll,XtNheight,&scrollsize,(String *)NULL);

/* map the current declination onto the scrollbar */

  /*
   * once again, we need to do a linear map onto the scrollbar length.  this
   * time, we want PI units to map onto the scrollbar size, so the slope of
   * this line is just (- (scrollsize / PI)) (the slope is again negative
   * because we want +90 to map to scrollbar position 0 and -90 to map to
   * the maximum scrollbar position).  b = y - mx, and we know that when
   * x is the initial declination (defined to be zero), we want y to be
   * (scrollsize / 2).  substituting again, b = (scrollsize / 2).  Note
   * that, unlike the RA case, an out-of-range value cannot occur here.
   */
  position = (- (scrollsize / PI)) * display.center_dec_rad + (scrollsize / 2);

/* calculate the new thumb percentage */
  pos = ((float)position / scrollsize);
  posptr = (XtArgVal *)&pos;

/* calculate the length of the thumb */
  length = (display.height * display.scale) / 360;
  lengthptr = (XtArgVal *)&length;

/* set the new position and length */
  XtVaSetValues(vert_scroll,XtNtopOfThumb,*posptr,XtNshown,*lengthptr,
		                                               (String *)NULL);

  return;
}



/* work procedure to handle -L (make big) option */

/* ARGSUSED */

static Boolean make_big_initial(client_data)

XtPointer client_data;

{
  Display *display;
  int screen;

/* save the old position and size of the toplevel widget */
  XtVaGetValues(toplevel,XtNx,&old_x,XtNy,&old_y,(String *)NULL);
  XtVaGetValues(toplevel,XtNwidth,&old_width,XtNheight,&old_height,
		                                               (String *)NULL);

/* change the toplevel widget's location and size to fill the display */
  display = XtDisplay(toplevel);
  screen = DefaultScreen(display);
  XtConfigureWidget(toplevel,0,0,DisplayWidth(display,screen),
		           DisplayHeight(display,screen) - VTWM_TITLEHEIGHT,1);

  return(True);
}



/* bring up the object type selection menu */

/* ARGSUSED */

static XtCallbackProc popup_cat_menu(menu_widget,client_data,call_data)

Widget menu_widget;
XtPointer client_data;
XtPointer call_data;

{
  Position x, y;
  Dimension width;

/* get the position of the menu widget */
  XtVaGetValues(menu_widget,XtNx,&x,XtNy,&y,(String *)NULL);

/* get the width of the menu widget */
  XtVaGetValues(menu_widget,XtNwidth,&width,(String *)NULL);

/* convert to root coordinates */
  XtTranslateCoords(toplevel,x,y,&x,&y);

/* place the submenu here */
  XtVaSetValues(catmenushell,XtNx,x + width / 2,XtNy,y,(String *)NULL);

/* pop up the submenu */
  XtPopup(catmenushell,XtGrabNonexclusive);

  return;
}



/* pop down the catalog selection menu */

/* ARGSUSED */

static XtActionProc popdown_cat_menu(w,event,params,num_params)

Widget w;
XEvent *event;
String *params;
Cardinal *num_params;

{
  boolean change_flag;
  struct cat_header *catalog;
  int n;
  Arg buttonargs[5];
  Boolean buttonstate;

/* pop down the object selection menu */
  XtPopdown(catmenushell);

/* see if anything actually changed in the object display */
  change_flag = FALSE;
  catalog = cat_list_head;
  while (catalog != (struct cat_header *)NULL) {
    /* get the current state of the toggle button */
    n = 0;
    XtSetArg(buttonargs[n],XtNstate,&buttonstate);                        n++;
    XtGetValues(catalog->catselect,buttonargs,n);

    /* make a note of any change */
    if (buttonstate != catalog->show_flag) {
      change_flag = TRUE;
      catalog->show_flag = buttonstate;
    }

    /* step to the next catalog */
    catalog = catalog->next_cat;
  }

/* if anything changed, update the sky display */
  if (change_flag)
    update_sky(&display);

  return;
}



/* pop down the object finder display */

/* ARGSUSED */

static XtActionProc popdown_find_display(w,event,params,num_params)

Widget w;
XEvent *event;
String *params;
Cardinal *num_params;

{
/* pop down the finder display shell */
  XtPopdown(findshell);

  return;
}



/* perform a display zoom */

/* ARGSUSED */

static XtCallbackProc display_zoom(w,client_data,call_data)

Widget w;
XtPointer client_data;
XtPointer call_data;

{
  float pixels;

/* calculate new pixel scale, changing by 20% in the appropriate direction */
  pixels = atof(scale_buffer);
  pixels += (int)client_data * (pixels * .2);

/* calculate the new scale of the display */
  display.scale = 1.0 / pixels;

/* update the scale buffer */
  sprintf(scale_buffer,"%7.3f",pixels);
  strcpy(old_scale_buffer,scale_buffer);

/* compute the new display width and height in radians */
  display.width_rad = (display.width * display.scale) / DEGREES_PER_RAD;
  display.height_rad = (display.height * display.scale) / DEGREES_PER_RAD;
#ifdef DEBUG
  printf("display dimensions = %lf, %lf radians\n",display.width_rad,
	                                                 display.height_rad);
#endif

/* update the sky display */
  update_sky(&display);

  return;
}



/* handle a jump scroll */

/* ARGSUSED */

static XtCallbackProc jump_scroll(w,client_data,position)

Widget w;
XtPointer client_data;
XtPointer position;

{
  update_thumbs();
  return;
}



/* handle an incremental scroll */

/* ARGSUSED */

static XtCallbackProc inc_scroll(w,client_data,position)

Widget w;
XtPointer client_data;
XtPointer position;

{
  int pos;
  Dimension width, height;
  int scrollsize;
  float floatval;
  XtArgVal *l_float;
  double distance_rad;

/* get the width and height of the scrollbar */
  XtVaGetValues(w,XtNwidth,&width,XtNheight,&height,(String *)NULL);

/* get the length of the scrollbar */
  if (w == vert_scroll)
    scrollsize = height;
  else if (w == horiz_scroll)
    scrollsize = width;
  else
    return;

/* extract position as a pixel count */
  pos = abs((int)position);

/* move the thumb to the new position */
  floatval = (float)pos / scrollsize;
  l_float = (XtArgVal *)&floatval;
  XtVaSetValues(w,XtNtopOfThumb,*l_float,(String *)NULL);

/* calculate the new center point of the display */

  /*
   * here's the scoop:  horizontal scrolling moves in RA only, and vertical
   * scrolling moves in dec only.  while the RA centerpoint may be modified,
   * by definition, the vertical scrollbar centerpoint is always 0d 0'; this
   * is to avoid ambiguity.  this particular restriction also means that the
   * vertical scrollbar only spans 180 degrees while the horizontal scrollbar
   * spans 360 degrees.  we calculate the distance from the center point,
   * convert that to radians, and then add to the current position.  we then
   * normalize that result to the range 0 <= x < 2 * pi for RA, but (for no
   * particular reason) we handle the normalization of the declination in
   * the rad_to_dec conversion routine.
   */

  /* modify the RA or dec, as appropriate */
  if (w == vert_scroll) {
    /* update the center point */
    distance_rad = - ((double)(pos - (scrollsize / 2)) / scrollsize) * PI;
    display.center_dec_rad = display.initial_dec_rad + distance_rad;
  }
  else if (w == horiz_scroll) {
    /* update the center point */
    distance_rad = - ((double)(pos - (scrollsize / 2)) / scrollsize) * 2 * PI;
    display.center_ra_rad = display.initial_ra_rad + distance_rad;
    if (display.center_ra_rad < 0)
      display.center_ra_rad += 2 * PI;
    else if (display.center_ra_rad >= (2 * PI))
      display.center_ra_rad -= 2 * PI;
    else
      ;
  }

/* update the display position */
  update_center_position();

/* update the sky display */
  update_sky(&display);

  return;
}



/* build the lists of displayable objects */

static void calculate_field(display)

struct display *display;

{
  struct id_node *id;
  struct cat_header *catalog;
  double ra_rad, dec_rad;
  int x, y;

/* calculate the limits of the display in RA and declination */
  calculate_limits(display);

/* calculate new lists of objects to display */
  catalog = cat_list_head;
  while (catalog != (struct cat_header *)NULL) {
    /* if there is an old list, free all the nodes */
    if (catalog->obj_list_head != (struct obj_node *)NULL) {
      (catalog->obj_list_rear)->next = freelist;
      freelist = catalog->obj_list_head;
      catalog->obj_list_head = (struct obj_node *)NULL;
      catalog->obj_list_rear = (struct obj_node *)NULL;
    }

    /* use this catalog only if it is active */
    if (catalog->show_flag)
      (*catalog->calculate_field)(display,catalog);

    /* step to the next catalog */
    catalog = catalog->next_cat;
  }

/* update the object identification list */
  id = id_list;
  while (id != (struct id_node *)NULL) {
    /* get the position of this object */
    (*(id->obj_catalog)->get_object_pos)(id->obj_index,&ra_rad,&dec_rad);

    /* calculate the new X position of the object */
    pos_rad_to_X(display,ra_rad,dec_rad,&x,&y);

    /* if the object is still in the display, update the ID */
    if ((x >= 0) && (x <= display->width) &&
	                                 (y >= 0) && (y <= display->height)) {
      /* flag this ID as displayable */
      id->show_flag = TRUE;

      /* update the ID's text position */
      id->x += x - id->obj_x;
      id->y += y - id->obj_y;

      /* update the ID's bounding box */
      id->ul_x += x - id->obj_x;
      id->ul_y += y - id->obj_y;
      id->lr_x += x - id->obj_x;
      id->lr_y += y - id->obj_y;
      
      /* and update the object's X position */
      id->obj_x = x;
      id->obj_y = y;
    }
    else
      /* flag this ID as not currently displayable */
      id->show_flag = FALSE;

    /* step to the next ID */
    id = id->next;
  }

  return;
}



/* add the given object to the display list if it's visible in the display */

void add_to_display_list(display,catalog,object_id,ra_rad,dec_rad)

struct display *display;
struct cat_header *catalog;
int object_id;
double ra_rad;
double dec_rad;

{
  int x, y;
  struct obj_node *object;

/* skip this object if it doesn't have a shot at being in the display */
  if (! within_limits(display,ra_rad,dec_rad))
    return;

/* convert the object's position to X coordinates */
  pos_rad_to_X(display,ra_rad,dec_rad,&x,&y);

/* see if the object fits in the display */
  if ((x >= 0) && (x <= display->width) &&
	                                 (y >= 0) && (y <= display->height)) {

#ifdef DEBUG
    printf("object %d from catalog %s falls within the display",object_id,
	                                                  catalog->menu_name);
    printf("at %lf, %lf radians\n",ra_rad,dec_rad);
#endif

/* get a node for this object */
    if (freelist == (struct obj_node *)NULL)
      object = (struct obj_node *)malloc(sizeof(struct obj_node));
    else {
      object = freelist;
      freelist = freelist->next;
    }

/* save the object ID (usually array position) of this object */
    object->array_pos = object_id;

/* save the X coordinates of this object */
    object->x = (Position)x;
    object->y = (Position)y;
#ifdef DEBUG
    printf("X coords of object are %d, %d\n",object->x,object->y);
#endif

/* link it into the display list at the rear */
    object->next = (struct obj_node *)NULL;
    if (catalog->obj_list_rear == (struct obj_node *)NULL)
      /* the list is empty */
      catalog->obj_list_head = catalog->obj_list_rear = object;
    else {
      (catalog->obj_list_rear)->next = object;
      catalog->obj_list_rear = object;
    }

#ifdef DEBUG
    printf("object linked into display list\n\n");
#endif
  }

  return;
}



/* change the center position of the display */

void update_display_pos(ra_rad,dec_rad)

double ra_rad, dec_rad;

{
  display.center_ra_rad = ra_rad;
  display.center_dec_rad = dec_rad;

  return;
}



/* build a dot interval table */

static int build_dot_intervals(interval_table,max_interval)

int interval_table[];
int max_interval;

{
  int divisor;
  int i;
  int j, k;

/* build a table of all divisors of the grid interval */
  for (i = 0, divisor = 1; divisor <= sqrt((double)max_interval); divisor++)
    if ((max_interval % divisor) == 0) {
      interval_table[i++] = divisor;
      interval_table[i++] = max_interval / divisor;
    }

/* sort the table */
  qsort((void *)interval_table,(size_t)i,sizeof(int),(int (*)())int_compare);

/* eliminate the duplicates from the table */
  j = k = 0;
  while (k < i)
    if (interval_table[k] == interval_table[k + 1])
      k++;
    else
      interval_table[j++] = interval_table[k++];

/* return the number of divisors */
  return(j);
}



/* build the grid interval table */

#if 0

static int build_grid_intervals(interval_table,max_interval)

int interval_table[];
int max_interval;

{
  int dividend, divisor;
  int i;

/* build a table of all divisors of one hour of time */
  for (i = 0, divisor = 1; divisor <= max_interval / 2; divisor++) {
    if (divisor < 1000)
      dividend = 1000;
    else if (divisor < (60 * 1000))
      dividend = 60 * 1000;
    else
      dividend = 60 * 60 * 1000;
    if ((dividend % divisor) == 0)
      interval_table[i++] = divisor;
  }

/* add the final divisor by hand */
  interval_table[i++] = max_interval;

/* return the number of divisors */
  return(i);
}

#else

static int build_grid_intervals(interval_table,max_interval)

int interval_table[];
int max_interval;

{
  return(build_dot_intervals(interval_table,max_interval));
}

#endif



/* compare two integers for qsort */

static int int_compare(int1,int2)

int *int1, *int2;

{
  return(*int1 - *int2);
}



/* display the command format */

static void printusage(argv)

char *argv[];

{
  printf("usage:  %s [-hLbcmfgspOBAFGKM]\n",argv[0]);
  exit(0);
}
